@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: DOMPrint.cpp,v 1.54 2003/01/28 18:30:56 peiyongz Exp $
+ * $Id: DOMPrint.cpp,v 1.58 2003/05/30 09:36:35 gareth Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -147,9 +147,6 @@
 //      The encoding we are to output in. If not set on the command line,
 //      then it is defaults to the encoding of the input XML file.
 //
-//  gMyEOLSequence
-//      The end of line sequence we are to output.
-//
 //  gSplitCdataSections
 //      Indicates whether split-cdata-sections is to be enabled or not.
 //
@@ -172,8 +169,7 @@ static bool                     gDoCreate              = false;
 
 static char*                    goutputfile            = 0;
 // options for DOMWriter's features
-static const XMLCh*             gOutputEncoding        = 0;
-static const XMLCh*             gMyEOLSequence         = 0;
+static XMLCh*                   gOutputEncoding        = 0;
 
 static bool                     gSplitCdataSections    = true;
 static bool                     gDiscardDefaultContent = true;
@@ -183,6 +179,11 @@ static bool                     gWriteBOM              = false;
 
 static XercesDOMParser::ValSchemes    gValScheme       = XercesDOMParser::Val_Auto;
 
+
+//	Prototypes for internally used functions
+void usage();
+
+
 // ---------------------------------------------------------------------------
 //
 //  Usage()
@@ -190,7 +191,7 @@ static XercesDOMParser::ValSchemes    gValScheme       = XercesDOMParser::Val_Au
 // ---------------------------------------------------------------------------
 void usage()
 {
-    cout << "\nUsage:\n"
+    XERCES_STD_QUALIFIER cout << "\nUsage:\n"
             "    DOMPrint [options] <XML file>\n\n"
             "This program invokes the DOM parser, and builds the DOM tree.\n"
             "It then asks the DOMWriter to serialize the DOM tree.\n"
@@ -203,7 +204,6 @@ void usage()
             "    -wenc=XXX   Use a particular encoding for output. Default is\n"
             "                the same encoding as the input XML file. UTF-8 if\n"
             "                input XML file has not XML declaration.\n"
-            "    -weol=xxx   Set the end of line sequence. Default set by DOMWriter\n"
             "    -wfile=xxx  Write to a file instead of stdout.\n"
             "    -wscs=xxx   Enable/Disable split-cdata-sections.      Default on\n"
             "    -wddc=xxx   Enable/Disable discard-default-content.   Default on\n"
@@ -215,7 +215,7 @@ void usage()
             "The parser has intrinsic support for the following encodings:\n"
             "    UTF-8, USASCII, ISO8859-1, UTF-16[BL]E, UCS-4[BL]E,\n"
             "    WINDOWS-1252, IBM1140, IBM037.\n"
-          <<  endl;
+          <<  XERCES_STD_QUALIFIER endl;
 }
 
 // ---------------------------------------------------------------------------
@@ -235,9 +235,9 @@ int main(int argC, char* argV[])
 
     catch(const XMLException &toCatch)
     {
-        cerr << "Error during Xerces-c Initialization.\n"
+        XERCES_STD_QUALIFIER cerr << "Error during Xerces-c Initialization.\n"
              << "  Exception message:"
-             << StrX(toCatch.getMessage()) << endl;
+             << StrX(toCatch.getMessage()) << XERCES_STD_QUALIFIER endl;
         return 1;
     }
 
@@ -277,7 +277,7 @@ int main(int argC, char* argV[])
                 gValScheme = XercesDOMParser::Val_Always;
             else
             {
-                cerr << "Unknown -v= value: " << parm << endl;
+                XERCES_STD_QUALIFIER cerr << "Unknown -v= value: " << parm << XERCES_STD_QUALIFIER endl;
                 XMLPlatformUtils::Terminate();
                 return 2;
             }
@@ -311,11 +311,6 @@ int main(int argC, char* argV[])
         {
              goutputfile =  &(argV[parmInd][7]);
         }
-         else if (!strncmp(argV[parmInd], "-weol=", 6))
-        {
-             // Get out the end of line
-             gMyEOLSequence = XMLString::transcode( &(argV[parmInd][6]) );
-        }			
          else if (!strncmp(argV[parmInd], "-wddc=", 6))
         {
             const char* const parm = &argV[parmInd][6];
@@ -326,7 +321,7 @@ int main(int argC, char* argV[])
 				gDiscardDefaultContent = false;
             else
             {
-                cerr << "Unknown -wddc= value: " << parm << endl;
+                XERCES_STD_QUALIFIER cerr << "Unknown -wddc= value: " << parm << XERCES_STD_QUALIFIER endl;
                 XMLPlatformUtils::Terminate();
                 return 2;
             }
@@ -342,7 +337,7 @@ int main(int argC, char* argV[])
 				gSplitCdataSections = false;
             else
             {
-                cerr << "Unknown -wscs= value: " << parm << endl;
+                XERCES_STD_QUALIFIER cerr << "Unknown -wscs= value: " << parm << XERCES_STD_QUALIFIER endl;
                 XMLPlatformUtils::Terminate();
                 return 2;
             }
@@ -357,7 +352,7 @@ int main(int argC, char* argV[])
 				gUseFilter = false;
             else
             {
-                cerr << "Unknown -wflt= value: " << parm << endl;
+                XERCES_STD_QUALIFIER cerr << "Unknown -wflt= value: " << parm << XERCES_STD_QUALIFIER endl;
                 XMLPlatformUtils::Terminate();
                 return 2;
             }
@@ -372,7 +367,7 @@ int main(int argC, char* argV[])
 				gFormatPrettyPrint = false;
             else
             {
-                cerr << "Unknown -wfpp= value: " << parm << endl;
+                XERCES_STD_QUALIFIER cerr << "Unknown -wfpp= value: " << parm << XERCES_STD_QUALIFIER endl;
                 XMLPlatformUtils::Terminate();
                 return 2;
             }
@@ -387,15 +382,15 @@ int main(int argC, char* argV[])
                 gWriteBOM = false;
             else
             {
-                cerr << "Unknown -wbom= value: " << parm << endl;
+                XERCES_STD_QUALIFIER cerr << "Unknown -wbom= value: " << parm << XERCES_STD_QUALIFIER endl;
                 XMLPlatformUtils::Terminate();
                 return 2;
             }
         }
          else
         {
-            cerr << "Unknown option '" << argV[parmInd]
-                 << "', ignoring it.\n" << endl;
+            XERCES_STD_QUALIFIER cerr << "Unknown option '" << argV[parmInd]
+                 << "', ignoring it.\n" << XERCES_STD_QUALIFIER endl;
         }
     }
 
@@ -438,8 +433,8 @@ int main(int argC, char* argV[])
 
     catch (const XMLException& e)
     {
-        cerr << "An error occurred during parsing\n   Message: "
-             << StrX(e.getMessage()) << endl;
+        XERCES_STD_QUALIFIER cerr << "An error occurred during parsing\n   Message: "
+             << StrX(e.getMessage()) << XERCES_STD_QUALIFIER endl;
         errorsOccured = true;
     }
 
@@ -448,18 +443,18 @@ int main(int argC, char* argV[])
         const unsigned int maxChars = 2047;
         XMLCh errText[maxChars + 1];
 
-        cerr << "\nDOM Error during parsing: '" << gXmlFile << "'\n"
-             << "DOMException code is:  " << e.code << endl;
+        XERCES_STD_QUALIFIER cerr << "\nDOM Error during parsing: '" << gXmlFile << "'\n"
+             << "DOMException code is:  " << e.code << XERCES_STD_QUALIFIER endl;
 
         if (DOMImplementation::loadDOMExceptionMsg(e.code, errText, maxChars))
-             cerr << "Message is: " << StrX(errText) << endl;
+             XERCES_STD_QUALIFIER cerr << "Message is: " << StrX(errText) << XERCES_STD_QUALIFIER endl;
 
         errorsOccured = true;
     }
 
     catch (...)
     {
-        cerr << "An error occurred during parsing\n " << endl;
+        XERCES_STD_QUALIFIER cerr << "An error occurred during parsing\n " << XERCES_STD_QUALIFIER endl;
         errorsOccured = true;
     }
 
@@ -476,8 +471,7 @@ int main(int argC, char* argV[])
             DOMImplementation *impl          = DOMImplementationRegistry::getDOMImplementation(tempStr);
             DOMWriter         *theSerializer = ((DOMImplementationLS*)impl)->createDOMWriter();
 
-            // set user specified end of line sequence and output encoding
-            theSerializer->setNewLine(gMyEOLSequence);
+            // set user specified output encoding
             theSerializer->setEncoding(gOutputEncoding);
 
             // plug in user's own filter
@@ -550,9 +544,9 @@ int main(int argC, char* argV[])
         }
         catch (XMLException& e)
         {
-            cerr << "An error occurred during creation of output transcoder. Msg is:"
-                << endl
-                << StrX(e.getMessage()) << endl;
+            XERCES_STD_QUALIFIER cerr << "An error occurred during creation of output transcoder. Msg is:"
+                << XERCES_STD_QUALIFIER endl
+                << StrX(e.getMessage()) << XERCES_STD_QUALIFIER endl;
             retval = 4;
         }
 
@@ -575,8 +569,7 @@ int main(int argC, char* argV[])
     // And call the termination method
     XMLPlatformUtils::Terminate();
 
-    delete (void *)gOutputEncoding;        // const problems.
-    delete (void *)gMyEOLSequence;         // const problems.
+    XMLString::release(&gOutputEncoding);
 
     return retval;
 }
