@@ -56,6 +56,18 @@
 
 /*
  * $Log: XMLDOMNamedNodeMap.h,v $
+ * Revision 1.5  2003/11/21 12:05:48  amassari
+ * Updated version to 2.4
+ *
+ * Revision 1.4  2003/10/21 21:21:32  amassari
+ * When the COM object is loaded by a late-binding engine (like WSH, or
+ * Visual Basic when the type library is not preloaded in the editor), the type
+ * library version stored in the resource must match the version specified in the
+ * IDispatchImpl template (defaulted to 1.0), or trying to invoke a method will fail
+ *
+ * Revision 1.3  2003/03/14 12:44:49  tng
+ * [Bug 17147] C++ namespace breaks build of XercesCOM DLL
+ *
  * Revision 1.2  2002/05/21 19:53:53  tng
  * DOM Reorganization: update include path for the old DOM interface in COM files
  *
@@ -85,9 +97,11 @@
 #include <xercesc/dom/deprecated/DOM_NamedNodeMap.hpp>
 #include "NodeContainerImpl.h"
 
+XERCES_CPP_NAMESPACE_USE
+
 class ATL_NO_VTABLE CXMLDOMNamedNodeMap :
 	public CComObjectRootEx<CComSingleThreadModel>,
-	public IDispatchImpl<IXMLDOMNamedNodeMap, &IID_IXMLDOMNamedNodeMap, &LIBID_Xerces>,
+	public IDispatchImpl<IXMLDOMNamedNodeMap, &IID_IXMLDOMNamedNodeMap, &LIBID_Xerces, 2, 40>,
 	public NodeContainerImpl<DOM_NamedNodeMap>,
 	public ISupportErrorInfo
 {
