@@ -57,7 +57,7 @@
  */
 
 /*
- * $Id: DOMImplementationImpl.hpp,v 1.9 2002/12/10 21:00:41 tng Exp $
+ * $Id: DOMImplementationImpl.hpp,v 1.14 2003/08/22 13:34:30 gareth Exp $
  */
 
 //
@@ -77,13 +77,15 @@ XERCES_CPP_NAMESPACE_BEGIN
 
 class XMLMsgLoader;
 
-class DOMImplementationImpl: public DOMImplementation,
+class DOMImplementationImpl: public XMemory,
+                             public DOMImplementation,
                              public DOMImplementationSource
 {
 private:
-    DOMImplementationImpl() {};
     DOMImplementationImpl(const DOMImplementationImpl & other) {};
     DOMImplementationImpl & operator = (const DOMImplementationImpl & other) {return *this;};
+protected:
+    DOMImplementationImpl() {};
 public:
     virtual ~DOMImplementationImpl() {};
     static DOMImplementationImpl*   getDOMImplementationImpl();
@@ -100,22 +102,25 @@ public:
                                                    const XMLCh *systemId);
     virtual DOMDocument*        createDocument(const XMLCh *namespaceURI,
                                                const XMLCh *qualifiedName,
-                                               DOMDocumentType *doctype);
+                                               DOMDocumentType *doctype,
+                                               MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
 
     // DOM Level 3
     virtual DOMImplementation*  getInterface(const XMLCh* feature);
 
     // Non-standard extension
-    virtual DOMDocument*        createDocument();
+    virtual DOMDocument*        createDocument(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
 
     // ------------------------------------------------------------
     // DOMImplementationLS Virtual interface
     // ------------------------------------------------------------
     // Introduced in DOM Level 3
     // Experimental - subject to change
-    virtual DOMBuilder*         createDOMBuilder(const short mode,
-                                                 const XMLCh* const schemaType);
-    virtual DOMWriter*          createDOMWriter();
+    virtual DOMBuilder*         createDOMBuilder(const short           mode,
+                                                 const XMLCh* const    schemaType,
+                                                 MemoryManager* const  manager = XMLPlatformUtils::fgMemoryManager,
+                                                 XMLGrammarPool* const gramPool = 0);
+    virtual DOMWriter*          createDOMWriter(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     virtual DOMInputSource*     createDOMInputSource();
 
     // ------------------------------------------------------------
