@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,21 @@
 
 /*
  * $Log: XMLAttDefList.hpp,v $
+ * Revision 1.7  2003/11/10 21:53:54  neilg
+ * add a means of statelessly traversing attribute lists.  mark the enumeration-based means as deprecated, since those are not stateless
+ *
+ * Revision 1.6  2003/10/20 11:47:30  gareth
+ * Constructor take memory manager and get method for it.
+ *
+ * Revision 1.5  2003/10/10 16:23:29  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.4  2003/05/16 21:36:55  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.3  2003/05/15 18:26:07  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.2  2002/11/04 15:00:21  tng
  * C++ Namespace Support.
  *
@@ -82,12 +97,14 @@
 #if !defined(XMLATTDEFLIST_HPP)
 #define XMLATTDEFLIST_HPP
 
-#include <xercesc/util/XMLEnumerator.hpp>
-#include <xercesc/util/XMLString.hpp>
-#include <xercesc/framework/XMLAttDef.hpp>
+#include <xercesc/util/XercesDefs.hpp>
+#include <xercesc/util/XMemory.hpp>
+#include <xercesc/internal/XSerializable.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
+class XMLAttDef;
+
 /**
  *  This class defines an abstract interface that all validators must support.
  *  When the scanner scans the attributes in a start tag, it must have a list
@@ -104,7 +121,8 @@ XERCES_CPP_NAMESPACE_BEGIN
  *  there are portability issues with deriving from a template class in a
  *  DLL. It does though provide a similar enumerator interface.
  */
-class XMLPARSER_EXPORT XMLAttDefList
+
+class XMLPARSER_EXPORT XMLAttDefList : public XSerializable, public XMemory
 {
 public:
     // -----------------------------------------------------------------------
@@ -120,6 +138,10 @@ public:
     // -----------------------------------------------------------------------
     //  The virtual interface
     // -----------------------------------------------------------------------
+
+    /** 
+     * @deprecated This method is not thread-safe.
+     */
     virtual bool hasMoreElements() const = 0;
     virtual bool isEmpty() const = 0;
     virtual XMLAttDef* findAttDef
@@ -142,20 +164,80 @@ public:
         const   XMLCh* const        attURI
         , const XMLCh* const        attName
     )   const = 0;
+
+    /** 
+     * @deprecated This method is not thread-safe.
+     */
     virtual XMLAttDef& nextElement() = 0;
+
+    /** 
+     * @deprecated This method is not thread-safe.
+     */
     virtual void Reset() = 0;
 
+    /**
+     * return total number of attributes in this list
+     */
+    virtual unsigned int getAttDefCount() const = 0;
+
+    /**
+     * return attribute at the index-th position in the list.
+     */
+    virtual XMLAttDef &getAttDef(unsigned int index) = 0;
+
+    /**
+     * return attribute at the index-th position in the list.
+     */
+    virtual const XMLAttDef &getAttDef(unsigned int index) const = 0;
+
+    /***
+     * Support for Serialization/De-serialization
+     ***/
+    DECL_XSERIALIZABLE(XMLAttDefList)
+
+
+    // -----------------------------------------------------------------------
+    //  Getter methods
+    // -----------------------------------------------------------------------
+
+    /** @name Getter methods */
+    //@{
+
+    /** Get the memory manager
+      *
+      * This method returns the configurable memory manager used by the
+      * element declaration for dynamic allocation/deacllocation.
+      *
+      * @return the memory manager
+      */
+    MemoryManager* getMemoryManager() const;
+
+    //@}
 
 protected :
     // -----------------------------------------------------------------------
     //  Hidden constructors and operators
     // -----------------------------------------------------------------------
-    XMLAttDefList();
+    XMLAttDefList(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
     XMLAttDefList(const XMLAttDefList&);
     XMLAttDefList& operator=(const XMLAttDefList&);
+
+
+private:
+    MemoryManager*      fMemoryManager;
 };
 
 
+
+// ---------------------------------------------------------------------------
+//  XMLAttDefList: Getter methods
+// ---------------------------------------------------------------------------
+
+inline MemoryManager* XMLAttDefList::getMemoryManager() const
+{
+    return fMemoryManager;
+}
+
 // ---------------------------------------------------------------------------
 //  XMLAttDefList: Constructors and Destructor
 // ---------------------------------------------------------------------------
@@ -167,7 +249,8 @@ inline XMLAttDefList::~XMLAttDefList()
 // ---------------------------------------------------------------------------
 //  XMLAttDefList: Hidden Constructors and Operators
 // ---------------------------------------------------------------------------
-inline XMLAttDefList::XMLAttDefList()
+inline XMLAttDefList::XMLAttDefList(MemoryManager* const manager):
+fMemoryManager(manager)
 {
 }
 
