@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 2002,2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@
 *  handler with the scanner. In these handler methods, appropriate DOM nodes
 *  are created and added to the DOM tree.
 *
-* $Id: AbstractDOMParser.cpp,v 1.37 2003/02/04 19:33:40 knoaman Exp $
+* $Id: AbstractDOMParser.cpp,v 1.57 2003/11/28 05:15:07 neilg Exp $
 *
 */
 
@@ -71,12 +71,13 @@
 #include <xercesc/parsers/AbstractDOMParser.hpp>
 #include <xercesc/internal/XMLScannerResolver.hpp>
 #include <xercesc/internal/ElemStack.hpp>
-#include <xercesc/sax/EntityResolver.hpp>
+//#include <xercesc/sax/EntityResolver.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/framework/XMLNotationDecl.hpp>
 #include <xercesc/framework/XMLValidator.hpp>
 #include <xercesc/util/IOException.hpp>
 #include <xercesc/dom/DOMImplementation.hpp>
+#include <xercesc/dom/DOMImplementationRegistry.hpp>
 #include <xercesc/dom/DOMElement.hpp>
 #include <xercesc/dom/impl/DOMAttrImpl.hpp>
 #include <xercesc/dom/DOMCDATASection.hpp>
@@ -96,13 +97,17 @@
 #include <xercesc/validators/common/ContentSpecNode.hpp>
 #include <xercesc/validators/common/GrammarResolver.hpp>
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 
 // ---------------------------------------------------------------------------
 //  AbstractDOMParser: Constructors and Destructor
 // ---------------------------------------------------------------------------
-AbstractDOMParser::AbstractDOMParser(XMLValidator* const valToAdopt) :
+AbstractDOMParser::AbstractDOMParser( XMLValidator* const   valToAdopt
+                                    , MemoryManager* const  manager
+                                    , XMLGrammarPool* const gramPool) :
 
   fCreateEntityReferenceNodes(true)
 , fIncludeIgnorableWhitespace(true)
@@ -111,6 +116,7 @@ AbstractDOMParser::AbstractDOMParser(XMLValidator* const valToAdopt) :
 , fCreateCommentNodes(true)
 , fDocumentAdoptedByUser(false)
 , fScanner(0)
+, fImplementationFeatures(0)
 , fCurrentParent(0)
 , fCurrentNode(0)
 , fCurrentEntity(0)
@@ -121,12 +127,20 @@ AbstractDOMParser::AbstractDOMParser(XMLValidator* const valToAdopt) :
 , fGrammarResolver(0)
 , fURIStringPool(0)
 , fValidator(valToAdopt)
+, fMemoryManager(manager)
+, fGrammarPool(gramPool)
+, fBufMgr(manager)
 , fInternalSubset(fBufMgr.bidOnBuffer())
+, fPSVIHandler(0)
 {
     try
     {
         initialize();
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
        cleanUp();
@@ -146,18 +160,17 @@ AbstractDOMParser::~AbstractDOMParser()
 void AbstractDOMParser::initialize()
 {
     //  Create grammar resolver and string pool to pass to the scanner
-    fGrammarResolver = new GrammarResolver;
-    fURIStringPool = new XMLStringPool;
+    fGrammarResolver = new (fMemoryManager) GrammarResolver(fGrammarPool, fMemoryManager);
+    fURIStringPool = fGrammarResolver->getStringPool();
 
     //  Create a scanner and tell it what validator to use. Then set us
     //  as the document event handler so we can fill the DOM document.
-    fScanner = XMLScannerResolver::getDefaultScanner(fValidator);
+    fScanner = XMLScannerResolver::getDefaultScanner(fValidator, fGrammarResolver, fMemoryManager);
     fScanner->setDocHandler(this);
     fScanner->setDocTypeHandler(this);
-    fScanner->setGrammarResolver(fGrammarResolver);
     fScanner->setURIStringPool(fURIStringPool);
 
-    fNodeStack = new ValueStackOf<DOMNode*>(64);
+    fNodeStack = new (fMemoryManager) ValueStackOf<DOMNode*>(64, fMemoryManager);
     this->reset();
 }
 
@@ -166,13 +179,15 @@ void AbstractDOMParser::cleanUp()
     if (fDocumentVector)
         delete fDocumentVector;
 
-    if (!fDocumentAdoptedByUser)
-        delete fDocument;
+    if (!fDocumentAdoptedByUser && fDocument)
+        fDocument->release();
 
     delete fNodeStack;
     delete fScanner;
     delete fGrammarResolver;
-    delete fURIStringPool;
+    // grammar pool *always* owns this
+    //delete fURIStringPool;
+    fMemoryManager->deallocate(fImplementationFeatures);
 
     if (fValidator)
         delete fValidator;
@@ -187,14 +202,13 @@ void AbstractDOMParser::reset()
     if (fDocument && !fDocumentAdoptedByUser) {
         if (!fDocumentVector) {
             // allocate the vector if not exists yet
-            fDocumentVector  = new RefVectorOf<DOMDocumentImpl>(10, true) ;
+            fDocumentVector  = new (fMemoryManager) RefVectorOf<DOMDocumentImpl>(10, true, fMemoryManager) ;
         }
         fDocumentVector->addElement(fDocument);
     }
 
     fDocument = 0;
     resetDocType();
-
     fCurrentParent   = 0;
     fCurrentNode     = 0;
     fCurrentEntity   = 0;
@@ -215,8 +229,8 @@ void AbstractDOMParser::resetPool()
     if (fDocumentVector)
         fDocumentVector->removeAllElements();
 
-     if (!fDocumentAdoptedByUser)
-       delete fDocument;
+    if (!fDocumentAdoptedByUser && fDocument)
+        fDocument->release();
 
     fDocument = 0;
 }
@@ -298,6 +312,11 @@ XMLCh* AbstractDOMParser::getExternalNoNamespaceSchemaLocation() const
     return fScanner->getExternalNoNamespaceSchemaLocation();
 }
 
+SecurityManager* AbstractDOMParser::getSecurityManager() const
+{
+    return fScanner->getSecurityManager();
+}
+
 bool AbstractDOMParser::getLoadExternalDTD() const
 {
     return fScanner->getLoadExternalDTD();
@@ -317,6 +336,18 @@ bool AbstractDOMParser::getStandardUriConformant() const
 // ---------------------------------------------------------------------------
 //  AbstractDOMParser: Setter methods
 // ---------------------------------------------------------------------------
+void AbstractDOMParser::setPSVIHandler(PSVIHandler* const handler)
+{
+    fPSVIHandler = handler;
+    if (fPSVIHandler) {
+        fScanner->setPSVIHandler(fPSVIHandler);        
+    }
+    else {
+        fScanner->setPSVIHandler(0);        
+    }
+}
+
+
 void AbstractDOMParser::setDoNamespaces(const bool newState)
 {
     fScanner->setDoNamespaces(newState);
@@ -370,6 +401,16 @@ void AbstractDOMParser::setExternalNoNamespaceSchemaLocation(const char* const n
     fScanner->setExternalNoNamespaceSchemaLocation(noNamespaceSchemaLocation);
 }
 
+void AbstractDOMParser::setSecurityManager(SecurityManager* const securityManager)
+{
+    // since this could impact various components, don't permit it to change
+    // during a parse
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    fScanner->setSecurityManager(securityManager);
+}
+
 void AbstractDOMParser::setLoadExternalDTD(const bool newState)
 {
     fScanner->setLoadExternalDTD(newState);
@@ -387,12 +428,17 @@ void AbstractDOMParser::setStandardUriConformant(const bool newState)
 
 void AbstractDOMParser::useScanner(const XMLCh* const scannerName)
 {
-    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner(scannerName, fValidator);
+    XMLScanner* tempScanner = XMLScannerResolver::resolveScanner
+    (
+        scannerName
+        , fValidator
+        , fGrammarResolver
+        , fMemoryManager
+    );
 
     if (tempScanner) {
 
         tempScanner->setParseSettings(fScanner);
-        tempScanner->setGrammarResolver(fGrammarResolver);
         tempScanner->setURIStringPool(fURIStringPool);
         delete fScanner;
         fScanner = tempScanner;
@@ -414,7 +460,10 @@ void AbstractDOMParser::parse(const InputSource& source)
         fScanner->scanDocument(source);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }    
     catch(...)
     {
         fParseInProgress = false;
@@ -434,7 +483,10 @@ void AbstractDOMParser::parse(const XMLCh* const systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -454,7 +506,10 @@ void AbstractDOMParser::parse(const char* const systemId)
         fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
-
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
         fParseInProgress = false;
@@ -676,15 +731,27 @@ void AbstractDOMParser::endElement( const   XMLElementDecl&     elemDecl
     fCurrentNode   = fCurrentParent;
     fCurrentParent = fNodeStack->pop();
 
-    //validation is performed after the startElement event so we have to associate the info here
+    // REVISIT:  delete when elementTypeInfo callback
+    // is implemented
     ((DOMElementImpl *)(fCurrentNode))->setTypeInfo(elemDecl.getDOMTypeInfoName(), elemDecl.getDOMTypeInfoUri());
-
-
     // If we've hit the end of content, clear the flag
     if (fNodeStack->empty())
         fWithinElement = false;
 }
 
+void AbstractDOMParser::elementTypeInfo( const   XMLCh * const  typeName
+                           , const XMLCh *const                 typeURI)
+{
+    /**** REVISIT:  enable when callback is implemented in scanners
+    // very tightly coupled to endElement().  The state of the
+    // object must not have changed between these calls!
+
+    //validation is performed after the startElement event so we have to associate the info here
+    ((DOMElementImpl *)(fCurrentNode))->setTypeInfo(typeName, typeURI);
+    *******/
+
+}
+
 
 void AbstractDOMParser::ignorableWhitespace(const   XMLCh* const    chars
                                     , const unsigned int    length
@@ -729,7 +796,11 @@ void AbstractDOMParser::resetDocument()
 
 void AbstractDOMParser::startDocument()
 {
-    fDocument = (DOMDocumentImpl *)DOMImplementation::getImplementation()->createDocument();
+    if(fImplementationFeatures == 0)
+        fDocument = (DOMDocumentImpl *)DOMImplementation::getImplementation()->createDocument(fMemoryManager);
+    else 
+        fDocument = (DOMDocumentImpl *)DOMImplementationRegistry::getDOMImplementation(fImplementationFeatures)->createDocument(fMemoryManager);
+
     // Just set the document as the current parent and current node
     fCurrentParent = fDocument;
     fCurrentNode   = fDocument;
@@ -806,8 +877,11 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
             const XMLAttr* oneAttrib = attrList.elementAt(index);
             unsigned int attrURIId = oneAttrib -> getURIId();
             namespaceURI = 0;
-            if (XMLString::equals(oneAttrib -> getName(), XMLNS))    //for xmlns=...
+            bool foundXMLNS = false;
+            if (XMLString::equals(oneAttrib -> getName(), XMLNS)) {   //for xmlns=...
                 attrURIId = fScanner->getXMLNSNamespaceId();
+                foundXMLNS = true;
+            }
             if (attrURIId != fScanner->getEmptyNamespaceId()) {  //TagName has a prefix
                 namespaceURI = fScanner->getURIText(attrURIId);   //get namespaceURI
             }
@@ -836,32 +910,7 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
 
             attr->setSpecified(oneAttrib->getSpecified());
 
-            XMLAttDef *attDef = 0;
-            if(defAttrs != 0)
-                attDef = defAttrs->findAttDef(attrURIId, oneAttrib->getQName());
-
-            if(attDef != 0) {
-                attr->setTypeInfo(attDef->getDOMTypeInfoName(), attDef->getDOMTypeInfoUri());
-                attDef->reset();
-            }
-            else {
-                const XMLCh *name = oneAttrib->getName();
-                if (XMLString::equals(oneAttrib->getPrefix(), XSI)) {
-                    if(XMLString::equals(name, SchemaSymbols::fgXSI_TYPE)) {
-                        attr->setTypeInfo(SchemaSymbols::fgDT_QNAME, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
-                    }
-                    else if(XMLString::equals(name, SchemaSymbols::fgATT_NILL)) {
-                        attr->setTypeInfo(SchemaSymbols::fgDT_BOOLEAN, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
-                    }
-                    else if(XMLString::equals(name, SchemaSymbols::fgXSI_NONAMESPACESCHEMALOCACTION)) {
-                        attr->setTypeInfo(SchemaSymbols::fgDT_ANYURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
-                    }
-                }
-                else {
-                    //for normal ns attrs
-                    attr->setTypeInfo(SchemaSymbols::fgDT_ANYURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
-                }
-            }
+            attr->setTypeInfo(oneAttrib->getValidatingTypeName(), oneAttrib->getValidatingTypeURI());
         }
     }
     else {    //DOM Level 1
@@ -889,14 +938,7 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
                 attr->fNode.isIdAttr(true);
             }
 
-            XMLAttDef *attDef = 0;
-            if(defAttrs != 0)
-                attDef = defAttrs->findAttDef(oneAttrib -> getURIId(), oneAttrib->getQName());
-
-            if(attDef != 0) {
-                attr->setTypeInfo(attDef->getDOMTypeInfoName(), attDef->getDOMTypeInfoUri());
-                attDef->reset();
-            }
+            attr->setTypeInfo(oneAttrib->getValidatingTypeName(), oneAttrib->getValidatingTypeURI());
 
         }
     }
@@ -908,9 +950,9 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
 
         DOMAttrImpl * insertAttr = 0;
 
-        while (defAttrs->hasMoreElements())
+        for(unsigned int i=0; i<defAttrs->getAttDefCount(); i++)
         {
-            attr = &defAttrs->nextElement();
+            attr = &defAttrs->getAttDef(i);
 
             const XMLAttDef::DefAttTypes defType = attr->getDefaultType();
             if ((defType == XMLAttDef::Default)
@@ -968,6 +1010,7 @@ void AbstractDOMParser::startElement(const  XMLElementDecl&         elemDecl
                     }
                 }
 
+                // REVISIT:  this won't work in multithreaded code...
                 insertAttr->setTypeInfo(attr->getDOMTypeInfoName(), attr->getDOMTypeInfoUri());
 
             }
@@ -1000,7 +1043,7 @@ void AbstractDOMParser::startEntityReference(const XMLEntityDecl& entDecl)
         entity->setActualEncoding(fScanner->getReaderMgr()->getCurrentEncodingStr());
     fCurrentEntity = entity;
 
-    DOMEntityReference *er = fDocument->createEntityReference(entName);
+    DOMEntityReference *er = fDocument->createEntityReferenceByParser(entName);
 
     //set the readOnly flag to false before appending node, will be reset in endEntityReference
     DOMEntityReferenceImpl *erImpl = (DOMEntityReferenceImpl *) er;
@@ -1285,9 +1328,9 @@ void AbstractDOMParser::endAttList
         DOMElement     *elem  = fDocument->createElement(elemDecl.getFullName());
         DOMElementImpl *elemImpl = (DOMElementImpl *) elem;
 
-		while (defAttrs->hasMoreElements())
+        for(unsigned int i=0; i<defAttrs->getAttDefCount(); i++)
         {
-            attr = &defAttrs->nextElement();
+            attr = &defAttrs->getAttDef(i);
             if (attr->getValue() != 0)
             {
                 if (fScanner->getDoNamespaces())
@@ -1311,7 +1354,10 @@ void AbstractDOMParser::endAttList
                         XMLCh temp[1000];
 
                         if (index > 999)
-                            prefix = new XMLCh[index+1];
+                            prefix = (XMLCh*) fMemoryManager->allocate
+                            (
+                                (index + 1) * sizeof(XMLCh)
+                            );//new XMLCh[index+1];
                         else
                             prefix = temp;
 
@@ -1323,7 +1369,7 @@ void AbstractDOMParser::endAttList
                             buf.append(XMLUni::fgXMLURIName);
 
                         if (index > 999)
-                            delete prefix;
+                            fMemoryManager->deallocate(prefix);//delete [] prefix;
                     }
                     else {
                         //   No prefix
