@@ -56,6 +56,18 @@
 
 /*
  * $Log: MsgCatalogLoader.cpp,v $
+ * Revision 1.15  2003/08/21 16:36:08  peiyongz
+ * use PlatformUtils::panic()
+ *
+ * Revision 1.14  2003/05/15 18:29:49  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.13  2003/03/09 16:42:05  peiyongz
+ * PanicHandler
+ *
+ * Revision 1.12  2003/02/20 20:20:11  peiyongz
+ * Allow set user specified error message file location in PlatformUtils::Initialize().
+ *
  * Revision 1.11  2002/12/12 16:46:18  peiyongz
  * MsgCatalog file name changed.
  *
@@ -150,39 +162,49 @@ MsgCatalogLoader::MsgCatalogLoader(const XMLCh* const msgDomain)
     &&  !XMLString::equals(msgDomain, XMLUni::fgXMLDOMMsgDomain)
     &&  !XMLString::equals(msgDomain, XMLUni::fgValidityDomain))
     {
-        XMLPlatformUtils::panic(XMLPlatformUtils::Panic_UnknownMsgDomain);
+        XMLPlatformUtils::panic(PanicHandler::Panic_UnknownMsgDomain);
     }
 
     // Prepare the path info
-    char catpath[1024];
-    memset(catpath, 0, sizeof catpath);
-    char *nlsHome = getenv("XERCESC_NLS_HOME");
+    char locationBuf[1024];
+    memset(locationBuf, 0, sizeof locationBuf);
+    const char *nlsHome = XMLMsgLoader::getNLSHome();
+    
     if (nlsHome)
     {
-        strcpy(catpath, nlsHome);
-        strcat(catpath, "/");
+    	strcpy(locationBuf, nlsHome);
+        strcat(locationBuf, "/");
     }
     else
     {
-        char *altHome = getenv("XERCESCROOT");
-        if (altHome)
+        nlsHome = getenv("XERCESC_NLS_HOME");
+        if (nlsHome)
         {
-            strcpy(catpath, altHome);
-            strcat(catpath, "/msg/");
+            strcpy(locationBuf, nlsHome);
+            strcat(locationBuf, "/");
         }
+        else
+        {
+            nlsHome = getenv("XERCESCROOT");
+            if (nlsHome)
+            {                       	
+                strcpy(locationBuf, nlsHome);
+                strcat(locationBuf, "/msg/");
+            }
+        }    
     }
-
+    
     // Prepare user-specified locale specific cat file
     char catuser[1024];
     memset(catuser, 0, sizeof catuser);
-    strcpy(catuser, catpath);
+    strcpy(catuser, locationBuf);
     strcat(catuser, "XercesMessages_");
     strcat(catuser, XMLMsgLoader::getLocale());
     strcat(catuser, ".cat");
         
     char catdefault[1024];
     memset(catdefault, 0, sizeof catdefault);
-    strcpy(catdefault, catpath);
+    strcpy(catdefault, locationBuf);
     strcat(catdefault, "XercesMessages_en_US.cat");
 
    /**
@@ -194,7 +216,7 @@ MsgCatalogLoader::MsgCatalogLoader(const XMLCh* const msgDomain)
     {
         // Probably have to call panic here
         printf("Could not open catalog:\n %s\n or %s\n", catuser, catdefault);
-        exit(1);
+        XMLPlatformUtils::panic(PanicHandler::Panic_CantLoadMsgDomain);
     }
 
     if (XMLString::equals(msgDomain, XMLUni::fgXMLErrDomain))
@@ -273,24 +295,24 @@ bool MsgCatalogLoader::loadMsg(const  XMLMsgLoader::XMLMsgId  msgToLoad
 
     bool bRet = false;
     if (repText1)
-        tmp1 = XMLString::transcode(repText1);
+        tmp1 = XMLString::transcode(repText1, XMLPlatformUtils::fgMemoryManager);
     if (repText2)
-        tmp2 = XMLString::transcode(repText2);
+        tmp2 = XMLString::transcode(repText2, XMLPlatformUtils::fgMemoryManager);
     if (repText3)
-        tmp3 = XMLString::transcode(repText3);
+        tmp3 = XMLString::transcode(repText3, XMLPlatformUtils::fgMemoryManager);
     if (repText4)
-        tmp4 = XMLString::transcode(repText4);
+        tmp4 = XMLString::transcode(repText4, XMLPlatformUtils::fgMemoryManager);
 
     bRet = loadMsg(msgToLoad, toFill, maxChars, tmp1, tmp2, tmp3, tmp4);
 
     if (tmp1)
-        delete [] tmp1;
+        XMLPlatformUtils::fgMemoryManager->deallocate(tmp1);//delete [] tmp1;
     if (tmp2)
-        delete [] tmp2;
+        XMLPlatformUtils::fgMemoryManager->deallocate(tmp2);//delete [] tmp2;
     if (tmp3)
-        delete [] tmp3;
+        XMLPlatformUtils::fgMemoryManager->deallocate(tmp3);//delete [] tmp3;
     if (tmp4)
-        delete [] tmp4;
+        XMLPlatformUtils::fgMemoryManager->deallocate(tmp4);//delete [] tmp4;
 
     return bRet;
 }
