@@ -56,6 +56,9 @@
 
 /*
  * $Log: Win32MsgLoader.hpp,v $
+ * Revision 1.3  2003/03/07 18:15:44  tng
+ * Return a reference instead of void for operator=
+ *
  * Revision 1.2  2002/11/04 15:10:41  tng
  * C++ Namespace Support.
  *
@@ -143,7 +146,7 @@ private :
     // -----------------------------------------------------------------------
     Win32MsgLoader();
     Win32MsgLoader(const Win32MsgLoader&);
-    void operator=(const Win32MsgLoader&);
+    Win32MsgLoader& operator=(const Win32MsgLoader&);
 
 
     // -----------------------------------------------------------------------
