@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: WinSockNetAccessor.cpp,v 1.2 2002/11/04 15:11:39 tng Exp $
+ * $Id: WinSockNetAccessor.cpp,v 1.3 2003/05/17 05:54:18 knoaman Exp $
  */
 
 
@@ -100,7 +100,7 @@ BinInputStream* WinSockNetAccessor::makeNew(const XMLURL&  urlSource)
         case XMLURL::HTTP:
         {
             BinHTTPURLInputStream* retStrm =
-                new BinHTTPURLInputStream(urlSource);
+                new (urlSource.getMemoryManager()) BinHTTPURLInputStream(urlSource);
             return retStrm;
             break;
         }
