@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: MacOSPlatformUtils.cpp,v 1.10 2002/12/06 16:41:10 tng Exp $
+ * $Id: MacOSPlatformUtils.cpp,v 1.19 2003/07/31 18:37:29 jberry Exp $
  */
 
 
@@ -93,6 +93,9 @@
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/Platforms/MacOS/MacOSPlatformUtils.hpp>
+#include <xercesc/util/Platforms/MacOS/MacCarbonFile.hpp>
+#include <xercesc/util/Platforms/MacOS/MacPosixFile.hpp>
+#include <xercesc/util/PanicHandler.hpp>
 
 #if (defined(XML_USE_INMEMORY_MSGLOADER) || defined(XML_USE_INMEM_MESSAGELOADER))
    #include <xercesc/util/MsgLoaders/InMemory/InMemMsgLoader.hpp>
@@ -116,12 +119,6 @@
 
 XERCES_CPP_NAMESPACE_BEGIN
 
-//----------------------------------------------------------------------------
-//	Local Constants
-//----------------------------------------------------------------------------
-const std::size_t kMaxStaticPathChars = 512;		// Size of our statically allocated path buffers
-
-
 //----------------------------------------------------------------------------
 // Function Prototypes
 //----------------------------------------------------------------------------
@@ -129,15 +126,13 @@ XMLCh*	ConvertColonToSlash(XMLCh* p, std::size_t charCount);
 XMLCh*	ConvertSlashToColon(XMLCh* p, std::size_t charCount);
 char*	ConvertSlashToColon(char* p, std::size_t charCount);
 
-XMLCh*	XMLCreateFullPathFromFSRef_X(const FSRef& startingRef);
-XMLCh*	XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef);
-XMLCh*	XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec);
-bool	XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref);
-bool	XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref);
-bool	XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec);
-
-std::size_t TranscodeUniCharsToUTF8(UniChar* src, char* dst, std::size_t srcCnt, std::size_t maxChars);
-std::size_t TranscodeUTF8ToUniChars(char* src, UniChar* dst, std::size_t maxChars);
+XMLCh*	XMLCreateFullPathFromFSRef_X(const FSRef& startingRef, MemoryManager* const manager);
+XMLCh*	XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef, MemoryManager* const manager);
+XMLCh*	XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec,
+                                            MemoryManager* const manager);
+bool	XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref, MemoryManager* const manager);
+bool	XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref, MemoryManager* const manager);
+bool	XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec, MemoryManager* const manager);
 
 
 //----------------------------------------------------------------------------
@@ -171,387 +166,30 @@ std::size_t TranscodeUTF8ToUniChars(char* src, UniChar* dst, std::size_t maxChar
 //
 // gHasMPAPIs
 //	 True if the Multiprocessing APIs are available.
+//
+// gUsePosixFiles
+//   True if we're using XMLMacPosixFile rather than XMLMacCarbonFile.
 //----------------------------------------------------------------------------
-static bool gFileSystemCompatible	= false;
-static bool gHasFSSpecAPIs			= false;
-static bool gHasFS2TBAPIs			= false;
-static bool gHasHFSPlusAPIs			= false;
-static bool gHasFSPathAPIs			= false;
-static bool gPathAPIsUsePosixPaths	= false;
-static bool gHasMPAPIs				= false;
-
-
-//----------------------------------------------------------------------------
-// XMLMacFile methods
-//----------------------------------------------------------------------------
-
-unsigned int
-XMLMacFile::currPos()
-{
-    OSErr err = noErr;
-    unsigned int pos = 0;
-
-    if (!mFileValid)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetCurPos);
-
-    if (gHasHFSPlusAPIs)
-    {
-        SInt64 bigPos = 0;
-        err = FSGetForkPosition(mFileRefNum, &bigPos);
-        if (err == noErr)
-            pos = bigPos;
-    }
-    else
-    {
-        long longPos;
-        err = GetFPos(mFileRefNum, &longPos);
-        if (err == noErr)
-            pos = longPos;
-    }
-
-    if (err != noErr)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetCurPos);
-
-    return pos;
-}
-
-
-void
-XMLMacFile::close()
-{
-    OSErr err = noErr;
-    if (!mFileValid)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
-
-    if (gHasHFSPlusAPIs)
-        err = FSCloseFork(mFileRefNum);
-    else
-        err = FSClose(mFileRefNum);
-
-    mFileValid = false;
-
-    if (err != noErr)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotCloseFile);
-}
-
-
-unsigned int
-XMLMacFile::size()
-{
-    OSErr err = noErr;
-    unsigned int len = 0;
-
-    if (!mFileValid)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetSize);
-
-    if (gHasHFSPlusAPIs)
-    {
-        SInt64 bigLen = 0;
-        err = FSGetForkSize(mFileRefNum, &bigLen);
-        if (err == noErr)
-            len = bigLen;
-    }
-    else
-    {
-        long longLen;
-        err = GetEOF(mFileRefNum, &longLen);
-        if (err == noErr)
-            len = longLen;
-    }
-
-    if (err != noErr)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetSize);
-
-    return len;
-}
-
-
-void
-XMLMacFile::openWithPermission(const XMLCh* const fileName, int macPermission)
-{
-    OSErr err = noErr;
-
-    if (mFileValid)
-        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
-
-    if (gHasHFSPlusAPIs)
-    {
-        FSRef ref;
-        if (!XMLParsePathToFSRef(fileName, ref))
-            err = fnfErr;
-
-        HFSUniStr255 forkName;
-        if (err == noErr)
-            err = FSGetDataForkName(&forkName);
-
-        if (err == noErr)
-            err = FSOpenFork(&ref, forkName.length, forkName.unicode, macPermission, &mFileRefNum);
-    }
-    else
-    {
-        FSSpec spec;
-        if (!XMLParsePathToFSSpec(fileName, spec))
-            err = fnfErr;
-
-        if (err == noErr)
-            err = FSpOpenDF(&spec, macPermission, &mFileRefNum);
-    }
-
-    if (err != noErr)
-        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
-
-    mFileValid = true;
-}
-
-
-void
-XMLMacFile::create(const XMLCh* const filePath)
-{
-    OSErr err = noErr;
-
-    //	Split path into directory and filename components
-    int posSlash = XMLString::lastIndexOf(filePath, '/', XMLString::stringLen(filePath) - 1);
-    int posName = (posSlash == -1) ? 0 : posSlash+1;
-
-    const XMLCh* namePtr = filePath + posName;
-    int nameLen = XMLString::stringLen(namePtr);
-
-    //	Make a temporary string of the directory
-    ArrayJanitor<XMLCh> dirPath(new XMLCh[namePtr - filePath + 1]);
-    XMLString::subString(dirPath.get(), filePath, 0, posName);
-
-    //	Create the file as appropriate for API set
-    if (gHasHFSPlusAPIs)
-    {
-    	//	HFS+
-        FSRef ref;
-
-        //	If we find an existing file, delete it
-        if (XMLParsePathToFSRef(filePath, ref))
-            FSDeleteObject(&ref);
-
-        //	Get a ref to the parent directory
-        if (!XMLParsePathToFSRef(dirPath.get(), ref))
-            err = fnfErr;
-
-        //	Create a new file using the unicode name
-        if (err == noErr)
-        {
-            UniChar uniName[256];
-            err = FSCreateFileUnicode(
-                    &ref,
-                    nameLen, CopyXMLChsToUniChars(namePtr, uniName, nameLen, sizeof(uniName)),
-                    0, NULL, NULL, NULL);
-        }
-    }
-    else
-    {
-    	//	HFS
-        FSSpec spec;
-
-        //	If we find an existing file, delete it
-        if (XMLParsePathToFSSpec(filePath, spec))
-            FSpDelete(&spec);
-
-        //	Get a spec to the parent directory
-        if (!XMLParsePathToFSSpec(dirPath.get(), spec))
-            err = fnfErr;
-
-        //	Check that the new name is not too long for HFS
-        if (err == noErr && nameLen > 31)
-            err = errFSNameTooLong;
-
-        if (err == noErr)
-        {
-            //	Transcode the unicode name to native encoding
-            ArrayJanitor<const char> nativeName(XMLString::transcode(namePtr));
-
-            // Make a partial pathname from our current spec (parent directory) to the new file
-            unsigned char name[31 * 2 + 1 * 2 + 1];
-            unsigned char* partial = &name[1];
-
-            *partial++ = ':';      			 // Partial leads with :
-            const unsigned char* specName = spec.name;	// Copy in spec name
-            for (int specCnt = *specName++; specCnt > 0; --specCnt)
-                *partial++ = *specName++;
-
-            *partial++ = ':';      			 // Path component separator
-            char c;
-            for (const char* p = nativeName.get(); (c = *p++) != 0; ) // Copy in new element
-                *partial++ = (c == ':') ? '/' : c;		// Convert : to /
-
-            name[0] = partial - &name[1];   // Set the pascal string name length
-
-            //	Update the spec: this will probably return fnfErr
-            //					 (since we just deleted any existing file)
-            err = FSMakeFSSpec(spec.vRefNum, spec.parID, name, &spec);
-
-            //	Create the file from the spec
-            err = FSpCreate(&spec, '??\??', 'TEXT', smSystemScript);
-        }
-    }
-
-    //	Fail if we didn't create the file
-    if (err != noErr)
-    {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
-        //ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
-    }
-}
-
-
-void
-XMLMacFile::open(const XMLCh* const fileName)
-{
-    openWithPermission(fileName, fsRdPerm);
-}
-
-
-void
-XMLMacFile::openFileToWrite(const XMLCh* const fileName)
-{
-    create(fileName);
-    openWithPermission(fileName, fsRdWrPerm);
-}
-
-
-unsigned int
-XMLMacFile::read(const unsigned int toRead, XMLByte* const toFill)
-{
-    unsigned int bytesRead = 0;
-    OSErr err = noErr;
-
-    if (!mFileValid)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
-
-    if (gHasHFSPlusAPIs)
-    {
-        ByteCount actualCount;
-        err = FSReadFork(mFileRefNum, fsFromMark, 0, toRead, toFill, &actualCount);
-        bytesRead = actualCount;
-    }
-    else
-    {
-        long byteCount = toRead;
-        err = FSRead(mFileRefNum, &byteCount, toFill);
-        bytesRead = byteCount;
-    }
-
-    if (err != noErr && err != eofErr)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotReadFromFile);
-
-    return bytesRead;
-}
-
-
-void
-XMLMacFile::write(const long byteCount, const XMLByte* const buffer)
-{
-    long bytesWritten = 0;
-    OSErr err = noErr;
-
-    if (byteCount <= 0 || buffer == NULL)
-        return;
-
-    if (!mFileValid)
-    {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
-    }
-
-    if (gHasHFSPlusAPIs)
-    {
-        ByteCount actualCount;
-        err = FSWriteFork(mFileRefNum, fsFromMark, 0, byteCount, buffer, &actualCount);
-        bytesWritten = actualCount;
-    }
-    else
-    {
-        long count = byteCount;
-        err = FSWrite(mFileRefNum, &count, buffer);
-        bytesWritten = count;
-    }
-
-    if ((err != noErr && err != eofErr) || (bytesWritten != byteCount))
-    {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
-    }
-}
-
-
-void
-XMLMacFile::reset()
-{
-    OSErr err = noErr;
-
-    if (!mFileValid)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotResetFile);
-
-    if (gHasHFSPlusAPIs)
-        err = FSSetForkPosition(mFileRefNum, fsFromStart, 0);
-    else
-        err = SetFPos(mFileRefNum, fsFromStart, 0);
-
-    if (err != noErr)
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotResetFile);
-}
-
-
-XMLMacFile::~XMLMacFile()
-{
-    if (mFileValid)
-        close();
-}
+bool gFileSystemCompatible	= false;
+bool gHasFSSpecAPIs			= false;
+bool gHasFS2TBAPIs			= false;
+bool gHasHFSPlusAPIs		= false;
+bool gHasFSPathAPIs			= false;
+bool gPathAPIsUsePosixPaths	= false;
+bool gHasMPAPIs				= false;
+bool gUsePosixFiles			= false;
 
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: The panic method
 // ---------------------------------------------------------------------------
-void
-XMLPlatformUtils::panic(const PanicReasons reason)
+void 
+XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 {
-    const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "A system synchronization error occurred";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Failed to complete platform dependent initialization";
-    else
-        reasonStr = "Unknown error source";
-
-    //
-    //  The default handling of panics is not very friendly.
-    //	To replace it with something more friendly, you'll need to:
-    //	- #define XML_USE_CUSTOM_PANIC_PROC
-    //	- Write, and link with, XMLCustomPanicProc
-    //	- Implement your panic handling within XMLCustomPanicProc.
-    //
-#if defined(XML_USE_CUSTOM_PANIC_PROC)
-    XMLCustomPanicProc(reason, reasonStr);
-#else
-    char text[256];
-
-    #if defined(XML_METROWERKS) || defined(_GLIBCPP_USE_C99)
-    std::snprintf(text, sizeof(text), "Xerces Panic Error: %s", reasonStr);
-    #else
-    std::sprintf(text, "Xerces Panic Error: %s", reasonStr);
-    #endif
-
-    Str255 pasText;
-    CopyCStringToPascal(text, pasText);
-    DebugStr(pasText);
-#endif
-
-    //	Life's got us down. Good-bye world.
-    std::exit(-1);
+    if (fgUserPanicHandler)
+		fgUserPanicHandler->panic(reason);
+	else
+		fgDefaultPanicHandler->panic(reason);
 }
 
 
@@ -580,8 +218,13 @@ XMLPlatformUtils::fileSize(const FileHandle theFile)
 FileHandle
 XMLPlatformUtils::openFile(const char* const fileName)
 {
-    ArrayJanitor<const XMLCh> xmlPath(XMLString::transcode(fileName));
-    return openFile(xmlPath.get());
+    // Check to make sure the file system is in a state where we can use it
+    if (!gFileSystemCompatible)
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
+    
+    return (file->open(fileName, false)) ? file.release() : NULL;
 }
 
 
@@ -592,18 +235,22 @@ XMLPlatformUtils::openFile(const XMLCh* const fileName)
     if (!gFileSystemCompatible)
         ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
 
-    Janitor<XMLMacAbstractFile> file(new XMLMacFile());
-    file->open(fileName);
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
 
-    return file.release();
+    return (file->open(fileName, false)) ? file.release() : NULL;
 }
 
 
 FileHandle
 XMLPlatformUtils::openFileToWrite(const char* const fileName)
 {
-    ArrayJanitor<const XMLCh> xmlPath(XMLString::transcode(fileName));
-    return openFileToWrite(xmlPath.get());
+    // Check to make sure the file system is in a state where we can use it
+    if (!gFileSystemCompatible)
+        ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
+
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
+
+    return (file->open(fileName, true)) ? file.release() : NULL;
 }
 
 
@@ -614,10 +261,9 @@ XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
     if (!gFileSystemCompatible)
         ThrowXML1(XMLPlatformUtilsException, XMLExcepts::File_CouldNotOpenFile, fileName);
 
-    Janitor<XMLMacAbstractFile> file(new XMLMacFile());
-    file->openFileToWrite(fileName);
+    Janitor<XMLMacAbstractFile> file(XMLMakeMacFile());
 
-    return file.release();
+    return (file->open(fileName, true)) ? file.release() : NULL;
 }
 
 
@@ -659,21 +305,26 @@ XMLPlatformUtils::resetFile(FileHandle theFile)
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
 XMLCh*
-XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                              MemoryManager* const manager)
 {
     XMLCh* path = NULL;
 
     if (gHasHFSPlusAPIs)
     {
         FSRef ref;
-        if (!XMLParsePathToFSRef(srcPath, ref) || (path = XMLCreateFullPathFromFSRef(ref)) == NULL)
-            path = XMLString::replicate(srcPath);
+        if (   !XMLParsePathToFSRef(srcPath, ref, manager)
+			|| (path = XMLCreateFullPathFromFSRef(ref, manager)) == NULL
+		   )
+            path = XMLString::replicate(srcPath, manager);
     }
     else
     {
         FSSpec spec;
-        if (!XMLParsePathToFSSpec(srcPath, spec) || (path = XMLCreateFullPathFromFSSpec(spec)) == NULL)
-            path = XMLString::replicate(srcPath);
+        if (   !XMLParsePathToFSSpec(srcPath, spec, manager)
+		    || (path = XMLCreateFullPathFromFSSpec(spec, manager)) == NULL
+		   )
+            path = XMLString::replicate(srcPath, manager);
     }
 
     return path;
@@ -687,103 +338,27 @@ XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
 }
 
 
-XMLCh*
-XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
-                                    , const XMLCh* const    relativePath)
-
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-    // Code from Windows largely unmodified for the Macintosh,
-    // with the exception of removing support for '\' path
-    // separator.
-    //
-    // Note that there is no support currently for Macintosh
-    // path separators ':'.
+	//	Get a newly allocated path to the current directory
+	FSSpec spec;
+	XMLCh* path =
+		(noErr == FSMakeFSSpec(0, 0, NULL, &spec))
+			? XMLCreateFullPathFromFSSpec(spec, manager)
+			: NULL;
+			
+    if (!path)
+		 ThrowXML(XMLPlatformUtilsException,
+	           XMLExcepts::File_CouldNotGetBasePathName);
+	           
+	return path;
+}
 
-    // Create a buffer as large as both parts and empty it
-    ArrayJanitor<XMLCh> tmpBuf(new XMLCh[XMLString::stringLen(basePath)
-        + XMLString::stringLen(relativePath)
-        + 2]);
-    tmpBuf[0] = 0;
 
-    //
-    //  If we have no base path, then just take the relative path as
-    //  is.
-    //
-    if (!basePath)
-    {
-        XMLString::copyString(tmpBuf.get(), relativePath);
-        return tmpBuf.release();
-    }
-
-    if (!*basePath)
-    {
-        XMLString::copyString(tmpBuf.get(), relativePath);
-        return tmpBuf.release();
-    }
-
-    const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-    if (*basePtr != chForwardSlash)
-    {
-        while ((basePtr >= basePath) && (*basePtr != chForwardSlash))
-            basePtr--;
-    }
-
-    // There is no relevant base path, so just take the relative part
-    if (basePtr < basePath)
-    {
-        XMLString::copyString(tmpBuf.get(), relativePath);
-        return tmpBuf.release();
-    }
-
-    //  We have some path part, so we need to check to see if we have to
-    //  weave any of the parts together.
-    const XMLCh* pathPtr = relativePath;
-
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a / or the null to mean anything
-        if ((*pathPtr != chForwardSlash) &&  *pathPtr)
-            break;
-
-        if (*pathPtr)
-            pathPtr++;
-
-        // If it's one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePath) && (*basePtr != chForwardSlash))
-                basePtr--;
-
-            // The base cannot provide enough levels, so it's in error/
-            if (basePtr < basePath)
-                ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_BasePathUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf.get();
-    const XMLCh* tmpPtr = basePath;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Orphan the buffer and return it
-    return tmpBuf.release();
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c) 
+{
+	//	We support only forward slash as a path delimiter
+    return (chForwardSlash == c);
 }
 
 
@@ -815,21 +390,13 @@ XMLPlatformUtils::getCurrentMillis()
 // library, which as of version 2.0 provides a nice set of primitives. Under
 // Mac OS X, the Multiprocessing library is a thin veneer over pthreads.
 //
-// For lack of any really good solutions, I've implemented these mutexes
+// For lack of any really universal solutions, I've implemented these mutexes
 // atop the Multiprocessing library. The critical regions employed here
 // support recursive behavior, which is required by Xerces.
 //
-// Please note that, despite this implementation, there are probably other
-// MacOS barriers to actually using Xerces in a multi-threaded environment.
-// Many other parts of your system and/or development environment may not
-// support pre-emption, even under Mac OS X. Examples may include the memory
-// allocator, the Unicode Converter or Utilities, and perhaps the file
-// system (though the FS is better with Multiprocessing as of System 9.0).
-//
-// These routines are provided somewhat speculatively, and with the philosphy
-// that this code, at least, shouldn't be the reason why multithreading
-// doesn't work. Compatibility of this library wrt the other areas described
-// above will be resolved in time.
+// Please note that, despite this implementation, there may be other barriers
+// to using Xerces in a multithreaded environment. The memory allocator
+// under Mac OS 9, for instance, is not thread safe.
 // ---------------------------------------------------------------------------
 
 void*
@@ -892,9 +459,7 @@ XMLPlatformUtils::unlockMutex(void* const mtxHandle)
 //  Miscellaneous synchronization methods
 //
 // Atomic manipulation is implemented atop routines that were traditionally
-// part of DriverServices, but are now apparently a part of Carbon. If this
-// selection proves to be invalid, similar routines in OpenTransport could
-// be used instead.
+// part of DriverServices, but are now a part of Carbon.
 // ---------------------------------------------------------------------------
 
 void*
@@ -949,7 +514,7 @@ XMLPlatformUtils::platformInit()
 	long value = 0;
 
 	//	Detect available functions
-
+	
     //	Look for file system services
     if (noErr == Gestalt(gestaltFSAttr, &value))
     {
@@ -963,14 +528,26 @@ XMLPlatformUtils::platformInit()
         gHasFSPathAPIs		= false;
         #endif
 
-        gPathAPIsUsePosixPaths = gHasFSPathAPIs && (value & (1 << gestaltFSUsesPOSIXPathsForConversion));
+        gPathAPIsUsePosixPaths = gHasFSPathAPIs
+								 && (value & (1 << gestaltFSUsesPOSIXPathsForConversion));
     }
 
-    //	Look for MP
-	gHasMPAPIs				= MPLibraryIsLoaded();
-
 	//	We require FSSpecs at a minimum
     gFileSystemCompatible	= gHasFSSpecAPIs;
+
+	//	Determine which file system to use (posix or carbon file access)
+	//	If we're using Metrowerks MSL, we surely don't want posix paths,
+	//	as MSL doesn't use them.
+	#if __MSL__ && (__MSL__ < 0x08000 || _MSL_CARBON_FILE_APIS)
+	gUsePosixFiles			= false;
+	#else
+	gUsePosixFiles			= noErr == Gestalt(gestaltSystemVersion, &value)
+							  && value >= 0x00001000
+							  ;
+	#endif
+
+    //	Look for MP
+	gHasMPAPIs				= MPLibraryIsLoaded();
 }
 
 
@@ -1149,8 +726,24 @@ ConvertSlashToColon(char* p, std::size_t charCount)
 }
 
 
+//	Factory method to make an appropriate subclass of XMLMacAbstractFile
+//	for our use
+XMLMacAbstractFile*
+XMLMakeMacFile(void)
+{
+	XMLMacAbstractFile* result = NULL;
+	
+	if (gUsePosixFiles)
+		result = new XMLMacPosixFile;
+	else
+		result = new XMLMacCarbonFile;
+		
+	return result;
+}
+
+
 bool
-XMLParsePathToFSRef(const XMLCh* const pathName, FSRef& ref)
+XMLParsePathToFSRef(const XMLCh* const pathName, FSRef& ref, MemoryManager* const manager)
 {
 	bool result;
 	
@@ -1170,16 +763,16 @@ XMLParsePathToFSRef(const XMLCh* const pathName, FSRef& ref)
 	//	classic case for this.
 		
 	if (TARGET_API_MAC_CARBON && gHasFSPathAPIs && gPathAPIsUsePosixPaths)
-		result = XMLParsePathToFSRef_X(pathName, ref);
+		result = XMLParsePathToFSRef_X(pathName, ref, manager);
 	else
-		result = XMLParsePathToFSRef_Classic(pathName, ref);
+		result = XMLParsePathToFSRef_Classic(pathName, ref, manager);
 		
 	return result;
 }
 
 
 bool
-XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref)
+XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref, MemoryManager* const manager)
 {
 	//	Parse Path to FSRef using FSPathMakeRef as available under
 	//	Mac OS X and CarbonLib 1.1 and greater.
@@ -1188,12 +781,12 @@ XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref)
 	std::size_t pathLen = XMLString::stringLen(pathName);
 
     //	Transcode XMLCh into UniChar
-	UniChar uniBuf[kMaxStaticPathChars];
-	CopyXMLChsToUniChars(pathName, uniBuf, pathLen, kMaxStaticPathChars);
+	UniChar uniBuf[kMaxMacStaticPathChars];
+	CopyXMLChsToUniChars(pathName, uniBuf, pathLen, kMaxMacStaticPathChars);
 	
 	//	Transcode Unicode to UTF-8
-	char utf8Buf[kMaxStaticPathChars];
-	pathLen = TranscodeUniCharsToUTF8(uniBuf, utf8Buf, pathLen, kMaxStaticPathChars-1);
+	char utf8Buf[kMaxMacStaticPathChars];
+	pathLen = TranscodeUniCharsToUTF8(uniBuf, utf8Buf, pathLen, kMaxMacStaticPathChars-1);
 	
 	//	Terminate the path
 	char* p = utf8Buf;
@@ -1204,7 +797,7 @@ XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref)
 	if (*p != '/')
 	{
 		//	Right justify the user path to make room for the pre-pended path
-		std::memmove(p + kMaxStaticPathChars - pathLen, p, pathLen);
+		std::memmove(p + kMaxMacStaticPathChars - pathLen, p, pathLen);
 				
 		//	Get the current directory
         FSSpec spec;
@@ -1215,14 +808,14 @@ XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref)
 		
 		//	Get pathname to the current directory
 		if (err == noErr)
-			err = FSRefMakePath(&ref, reinterpret_cast<UInt8*>(p), kMaxStaticPathChars - pathLen - 1);	// leave room for one '/'
+			err = FSRefMakePath(&ref, reinterpret_cast<UInt8*>(p), kMaxMacStaticPathChars - pathLen - 1);	// leave room for one '/'
 		std::size_t prefixLen = std::strlen(p);
 			
 		//	Now munge the two paths back together
 		if (err == noErr)
 		{
 			p[prefixLen++] = '/';
-			std::memmove(p + prefixLen, p + kMaxStaticPathChars - pathLen, pathLen);
+			std::memmove(p + prefixLen, p + kMaxMacStaticPathChars - pathLen, pathLen);
 		}
 		
 		//	We now have a path from an absolute starting point
@@ -1239,7 +832,7 @@ XMLParsePathToFSRef_X(const XMLCh* const pathName, FSRef& ref)
 
 
 bool
-XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
+XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref, MemoryManager* const manager)
 {
 	//	Parse Path to FSRef by stepping manually through path components.
 	
@@ -1306,7 +899,7 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
             err = FSpMakeFSRef(&spec, &ref);
     }
 
-    // ref now refers to the a parent directory: parse the rest of the path
+    // ref now refers to the parent directory: parse the rest of the path
     while (err == noErr && *p)
     {
         switch (*p)
@@ -1371,7 +964,8 @@ XMLParsePathToFSRef_Classic(const XMLCh* const pathName, FSRef& ref)
 
 
 bool
-XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec)
+XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec,
+                            MemoryManager* const manager)
 {
 	//	Parse Path to an FSSpec
 
@@ -1385,7 +979,7 @@ XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec)
 	{
 		//	Parse to a ref
 		FSRef ref;
-		result = XMLParsePathToFSRef(pathName, ref);
+		result = XMLParsePathToFSRef(pathName, ref, manager);
 		
 		//	Down convert to a spec
 		if (result)
@@ -1398,7 +992,7 @@ XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec)
 								));
 	}
 	else
-		result = XMLParsePathToFSSpec_Classic(pathName, spec);
+		result = XMLParsePathToFSSpec_Classic(pathName, spec, manager);
 		
 	//	Return true on success
 	return result;
@@ -1406,13 +1000,14 @@ XMLParsePathToFSSpec(const XMLCh* const pathName, FSSpec& spec)
 
 
 bool
-XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
+XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec,
+                            MemoryManager* const manager)
 {
 	//	Manually parse the path using FSSpec APIs.
 	
     //	Transcode the path into ascii
-    const char* p = XMLString::transcode(pathName);
-    ArrayJanitor<const char> janPath(p);
+    const char* p = XMLString::transcode(pathName, manager);
+    ArrayJanitor<const char> janPath(p, manager);
     const char* pEnd;
     std::size_t segLen;
 
@@ -1564,7 +1159,8 @@ XMLParsePathToFSSpec_Classic(const XMLCh* const pathName, FSSpec& spec)
 
 
 XMLCh*
-XMLCreateFullPathFromFSRef(const FSRef& startingRef)
+XMLCreateFullPathFromFSRef(const FSRef& startingRef,
+                            MemoryManager* const manager)
 {
 	XMLCh* result = NULL;
 	
@@ -1582,39 +1178,41 @@ XMLCreateFullPathFromFSRef(const FSRef& startingRef)
 	//	makes paths with ':' separators, which really confuses us!
 	
 	if (TARGET_API_MAC_CARBON && gHasFSPathAPIs && gPathAPIsUsePosixPaths)
-		result = XMLCreateFullPathFromFSRef_X(startingRef);
+		result = XMLCreateFullPathFromFSRef_X(startingRef, manager);
 	else
-		result = XMLCreateFullPathFromFSRef_Classic(startingRef);
+		result = XMLCreateFullPathFromFSRef_Classic(startingRef, manager);
 		
 	return result;
 }
 
 
 XMLCh*
-XMLCreateFullPathFromFSRef_X(const FSRef& startingRef)
+XMLCreateFullPathFromFSRef_X(const FSRef& startingRef,
+                            MemoryManager* const manager)
 {
 	//	Create the path using FSRefMakePath as available on Mac OS X
 	//	and under CarbonLib 1.1 and greater.
 	OSStatus err = noErr;
 	
 	//	Make the path in utf8 form
-	char utf8Buf[kMaxStaticPathChars];
+	char utf8Buf[kMaxMacStaticPathChars];
 	utf8Buf[0] = '\0';
 	
 	if (err == noErr)
-		err = FSRefMakePath(&startingRef, reinterpret_cast<UInt8*>(utf8Buf), kMaxStaticPathChars);
+		err = FSRefMakePath(&startingRef, reinterpret_cast<UInt8*>(utf8Buf), kMaxMacStaticPathChars);
 		
 	//	Bail if path conversion failed
 	if (err != noErr)
 		return NULL;
 	
 	//	Transcode into UniChars
-	UniChar uniBuf[kMaxStaticPathChars];
-	std::size_t pathLen = TranscodeUTF8ToUniChars(utf8Buf, uniBuf, kMaxStaticPathChars-1);
+	UniChar uniBuf[kMaxMacStaticPathChars];
+	std::size_t pathLen = TranscodeUTF8ToUniChars(utf8Buf, uniBuf, kMaxMacStaticPathChars-1);
 	uniBuf[pathLen++] = 0;
 	
 	//	Transcode into a dynamically allocated buffer of XMLChs
-	ArrayJanitor<XMLCh> result(new XMLCh[pathLen]);
+	ArrayJanitor<XMLCh> result((XMLCh*) manager->allocate(pathLen * sizeof(XMLCh))/*new XMLCh[pathLen]*/,
+			manager);
 	if (result.get() != NULL)
 		CopyUniCharsToXMLChs(uniBuf, result.get(), pathLen, pathLen);
 		
@@ -1623,7 +1221,8 @@ XMLCreateFullPathFromFSRef_X(const FSRef& startingRef)
 
 
 XMLCh*
-XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef)
+XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef,
+                            MemoryManager* const manager)
 {
 	//	Manually create the path using FSRef APIs.
     OSStatus err = noErr;
@@ -1631,8 +1230,8 @@ XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef)
     HFSUniStr255 name;
     FSRef ref = startingRef;
 
-    XMLCh buf[kMaxStaticPathChars];
-    std::size_t bufPos   = kMaxStaticPathChars;
+    XMLCh buf[kMaxMacStaticPathChars];
+    std::size_t bufPos   = kMaxMacStaticPathChars;
     std::size_t bufCnt   = 0;
 
     ArrayJanitor<XMLCh> result(NULL);
@@ -1658,7 +1257,11 @@ XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef)
 			// name plus separator, dump it to the dynamic result buffer.
 			if (bufPos < (std::size_t)name.length + 1)
 			{
-				ArrayJanitor<XMLCh> temp(new XMLCh[bufCnt + resultLen]);
+				ArrayJanitor<XMLCh> temp
+                (
+                    (XMLCh*) manager->allocate((bufCnt + resultLen) * sizeof(XMLCh)) //new XMLCh[bufCnt + resultLen]
+                    , manager
+                );
 				
 				// Copy in the static buffer
 				std::memcpy(temp.get(), &buf[bufPos], bufCnt * sizeof(XMLCh));
@@ -1670,21 +1273,27 @@ XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef)
 				result.reset(temp.release());
 				resultLen += bufCnt;
 				
-				bufPos = kMaxStaticPathChars;
+				bufPos = kMaxMacStaticPathChars;
 				bufCnt = 0;
 			}
 			
 			// Prepend our new name and a '/'
 			bufPos -= name.length;
-			ConvertSlashToColon(CopyUniCharsToXMLChs(name.unicode, &buf[bufPos], name.length, name.length), name.length);
+			ConvertSlashToColon(
+				CopyUniCharsToXMLChs(name.unicode, &buf[bufPos], name.length, name.length),
+				name.length);
 			buf[--bufPos] = L'/';
 			bufCnt += (name.length + 1);
 		}
 	}
 	while (err == noErr && catalogInfo.parentDirID != fsRtParID);
 	
-	// Composite existing buffer with any previous result buffer
-	ArrayJanitor<XMLCh> final(new XMLCh[bufCnt + resultLen]);
+	// Composite existing buffer + any previous result buffer
+	ArrayJanitor<XMLCh> final
+    (
+        (XMLCh*) manager->allocate((bufCnt + resultLen) * sizeof(XMLCh))//new XMLCh[bufCnt + resultLen]
+        , manager
+    );
 	
 	// Copy in the static buffer
 	std::memcpy(final.get(), &buf[bufPos], bufCnt * sizeof(XMLCh));
@@ -1698,7 +1307,8 @@ XMLCreateFullPathFromFSRef_Classic(const FSRef& startingRef)
 
 
 XMLCh*
-XMLCreateFullPathFromFSSpec(const FSSpec& startingSpec)
+XMLCreateFullPathFromFSSpec(const FSSpec& startingSpec,
+                            MemoryManager* const manager)
 {
 	XMLCh* result = NULL;
 	
@@ -1716,12 +1326,12 @@ XMLCreateFullPathFromFSSpec(const FSSpec& startingSpec)
 			
 		//	Create the path
 		if (err == noErr)
-			result = XMLCreateFullPathFromFSRef(ref);
+			result = XMLCreateFullPathFromFSRef(ref, manager);
 	}
 	else
 	{
 		//	Create using FSSpecs only
-		result = XMLCreateFullPathFromFSSpec_Classic(startingSpec);
+		result = XMLCreateFullPathFromFSSpec_Classic(startingSpec, manager);
 	}
 		
 	return result;
@@ -1729,14 +1339,15 @@ XMLCreateFullPathFromFSSpec(const FSSpec& startingSpec)
 
 
 XMLCh*
-XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
+XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec,
+                                    MemoryManager* const manager)
 {
 	//	Manually create the path using FSSpec APIs.
     OSStatus err = noErr;
     FSSpec spec = startingSpec;
 
-    char buf[kMaxStaticPathChars];
-    std::size_t bufPos   = kMaxStaticPathChars;
+    char buf[kMaxMacStaticPathChars];
+    std::size_t bufPos   = kMaxMacStaticPathChars;
     std::size_t bufCnt   = 0;
 
     ArrayJanitor<char> result(NULL);
@@ -1763,7 +1374,11 @@ XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
 			// name plus separator, dump it to the dynamic result buffer.
 			if (bufPos < nameLen + 1)
 			{
-				ArrayJanitor<char> temp(new char[bufCnt + resultLen]);
+				ArrayJanitor<char> temp
+                (
+                    (char*) manager->allocate((bufCnt + resultLen) * sizeof(char))//new char[bufCnt + resultLen]
+                    , manager
+                );
 				
 				// Copy in the static buffer
 				std::memcpy(temp.get(), &buf[bufPos], bufCnt);
@@ -1775,7 +1390,7 @@ XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
 				result.reset(temp.release());
 				resultLen += bufCnt;
 				
-				bufPos = kMaxStaticPathChars;
+				bufPos = kMaxMacStaticPathChars;
 				bufCnt = 0;
 			}
 			
@@ -1795,7 +1410,11 @@ XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
 	while (err == noErr && spec.parID != fsRtParID);
 	
 	// Composite existing buffer with any previous result buffer
-	ArrayJanitor<char> final(new char[bufCnt + resultLen]);
+	ArrayJanitor<char> final
+    (
+        (char*) manager->allocate((bufCnt + resultLen) * sizeof(char))//new char[bufCnt + resultLen]
+        , manager
+    );
 	
 	// Copy in the static buffer
 	std::memcpy(final.get(), &buf[bufPos], bufCnt);
@@ -1805,12 +1424,12 @@ XMLCreateFullPathFromFSSpec_Classic(const FSSpec& startingSpec)
 		std::memcpy(final.get() + bufCnt, result.get(), resultLen);
 
     // Cleanup and transcode to unicode
-    return XMLString::transcode(final.get());
+    return XMLString::transcode(final.get(), manager);
 }
 
 
 std::size_t
-TranscodeUniCharsToUTF8(UniChar* src, char* dst, std::size_t srcCnt, std::size_t maxChars)
+TranscodeUniCharsToUTF8(const UniChar* src, char* dst, std::size_t srcCnt, std::size_t maxChars)
 {
 	std::size_t result = 0;
 	
@@ -1853,7 +1472,7 @@ TranscodeUniCharsToUTF8(UniChar* src, char* dst, std::size_t srcCnt, std::size_t
 
 
 std::size_t
-TranscodeUTF8ToUniChars(char* src, UniChar* dst, std::size_t maxChars)
+TranscodeUTF8ToUniChars(const char* src, UniChar* dst, std::size_t maxChars)
 {
 	std::size_t result = 0;
 	
@@ -1894,4 +1513,6 @@ TranscodeUTF8ToUniChars(char* src, UniChar* dst, std::size_t maxChars)
 	return result;
 }
 
+#include <xercesc/util/LogicalPath.c>
+
 XERCES_CPP_NAMESPACE_END
