@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: SolarisPlatformUtils.cpp,v 1.9 2003/01/03 16:02:12 tng Exp $
+ * $Id: SolarisPlatformUtils.cpp,v 1.17 2003/11/17 16:53:34 peiyongz Exp $
  */
 
 
@@ -82,6 +82,9 @@
 #include    <link.h>
 #include    <limits.h>
 #include    <dlfcn.h>
+#include    <sys/types.h>
+#include    <sys/stat.h>
+#include    <fcntl.h>
 #include    <xercesc/util/Janitor.hpp>
 #include    <xercesc/util/PlatformUtils.hpp>
 #include    <xercesc/util/RuntimeException.hpp>
@@ -89,6 +92,8 @@
 #include    <xercesc/util/XMLString.hpp>
 #include    <xercesc/util/XMLUniDefs.hpp>
 #include    <xercesc/util/XMLUni.hpp>
+#include    <xercesc/util/PanicHandler.hpp>
+#include    <xercesc/util/OutOfMemoryException.hpp>
 
 #if defined (XML_USE_ICU_TRANSCODER)
     #include <xercesc/util/Transcoders/ICU/ICUTransService.hpp>
@@ -149,9 +154,13 @@ XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
         retVal = new InMemMsgLoader(msgDomain);
 #endif
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch(...)
     {
-        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+        panic(PanicHandler::Panic_CantLoadMsgDomain);
     }
     return retVal;
 }
@@ -184,28 +193,9 @@ XMLTransService* XMLPlatformUtils::makeTransService()
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: The panic method
 // ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
+void XMLPlatformUtils::panic(const PanicHandler::PanicReasons reason)
 {
-
-    const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "Cannot synchronize system or mutex";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Cannot initialize the system or mutex";
-
-    fprintf(stderr, "%s\n", reasonStr);
-
-    exit(-1);
+    fgUserPanicHandler? fgUserPanicHandler->panic(reason) : fgDefaultPanicHandler->panic(reason);	
 }
 
 // ---------------------------------------------------------------------------
@@ -214,7 +204,7 @@ void XMLPlatformUtils::panic(const PanicReasons reason)
 unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
 {
     // Get the current position
-    int curPos = ftell( (FILE*)theFile);
+    int curPos = tell( (int)theFile);
     if (curPos == -1)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotGetSize);
@@ -224,7 +214,7 @@ unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
 
 void XMLPlatformUtils::closeFile(FileHandle theFile)
 {
-    if (fclose((FILE*) theFile))
+    if (close((int) theFile))
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotCloseFile);
 }
@@ -232,23 +222,23 @@ void XMLPlatformUtils::closeFile(FileHandle theFile)
 unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
 {
     // Get the current position
-    long  int curPos = ftell((FILE*) theFile);
+    long  int curPos = tell((int) theFile);
     if (curPos == -1)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotGetCurPos);
 
     // Seek to the end and save that value for return
-    if (fseek( (FILE*) theFile, 0, SEEK_END) )
+    if (lseek( (int) theFile, 0, SEEK_END) == (off_t)-1)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotSeekToEnd);
 
-    long int retVal = ftell((FILE*) theFile);
+    long int retVal = tell((int) theFile);
     if (retVal == -1)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotSeekToEnd);
 
     // And put the pointer back
-    if (fseek((FILE*) theFile, curPos, SEEK_SET))
+    if (lseek((int) theFile, curPos, SEEK_SET) == (off_t)-1)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotSeekToPos);
 
@@ -257,34 +247,34 @@ unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
 
 FileHandle XMLPlatformUtils::openFile(const char* const fileName)
 {
-    FileHandle retVal = (FILE*)fopen( fileName , "rb" );
+    int retVal = open( fileName , O_RDONLY );
 
-    if (retVal == NULL)
+    if (retVal == -1)
         return 0;
-    return retVal;
+    return (FileHandle)retVal;
 }
 
 FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
-    FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
+    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
+    int retVal = open( tmpFileName , O_RDONLY );
 
-    if (retVal == NULL)
+    if (retVal == -1)
         return 0;
-    return retVal;
+    return (FileHandle)retVal;
 }
 
 FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
 {
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
-    return fopen( tmpFileName , "wb" );
+    const char* tmpFileName = XMLString::transcode(fileName, fgMemoryManager);
+    ArrayJanitor<char> janText((char*)tmpFileName, fgMemoryManager);
+    return (FileHandle)open( tmpFileName , O_WRONLY | O_CREAT );
 }
 
 FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
 {
-    return fopen( fileName , "wb" );
+    return (FileHandle)open( fileName , O_WRONLY | O_CREAT );
 }
 
 unsigned int
@@ -292,10 +282,10 @@ XMLPlatformUtils::readFileBuffer(FileHandle              theFile
                                , const unsigned int      toRead
                                , XMLByte* const          toFill)
 {
-    size_t noOfItemsRead =
-               fread((void*) toFill, 1, toRead, (FILE*) theFile);
+    ssize_t noOfItemsRead =
+      read((int) theFile, (void*) toFill, toRead);
 
-    if(ferror((FILE*) theFile))
+    if(noOfItemsRead == -1)
     {
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotReadFromFile);
@@ -315,13 +305,13 @@ XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
         return;
 
     const XMLByte* tmpFlush = (const XMLByte*) toFlush;
-    size_t bytesWritten = 0;
+    ssize_t bytesWritten = 0;
 
     while (true)
     {
-        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+      bytesWritten=write((int)theFile, (const void *)tmpFlush, toWrite);
 
-        if(ferror((FILE*)theFile))
+        if(bytesWritten == -1)
         {
             ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
         }
@@ -342,7 +332,7 @@ XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
     // Seek to the start of the file
-    if (fseek((FILE*) theFile, 0, SEEK_SET))
+    if (lseek((int) theFile, 0, SEEK_SET) == -1)
         ThrowXML(XMLPlatformUtilsException,
                  XMLExcepts::File_CouldNotResetFile);
 }
@@ -351,27 +341,27 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath,
+                                     MemoryManager* const manager)
 {
     //
     //  NOTE: THe path provided has always already been opened successfully,
     //  so we know that its not some pathological freaky path. It comes in
     //  in native format, and goes out as Unicode always
     //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
+    char* newSrc = XMLString::transcode(srcPath, fgMemoryManager);
+    ArrayJanitor<char> janText(newSrc, fgMemoryManager);
 
     // Use a local buffer that is big enough for the largest legal path
-    char *absPath = new char[PATH_MAX];
-    ArrayJanitor<char> janText2(absPath);
+    char absPath[PATH_MAX + 1];
     //get the absolute path
-    char* retPath = realpath(newSrc, absPath);
+    char* retPath = realpath(newSrc, &absPath[0]);
 
     if (!retPath)
     {
         ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
     }
-    return XMLString::transcode(absPath);
+    return XMLString::transcode(absPath, manager);
 }
 
 bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
@@ -392,116 +382,24 @@ bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
     return true;
 }
 
-XMLCh* XMLPlatformUtils::weavePaths
-    (
-        const   XMLCh* const    basePath
-        , const XMLCh* const    relativePath
-    )
+XMLCh* XMLPlatformUtils::getCurrentDirectory(MemoryManager* const manager)
 {
-// Create a buffer as large as both parts and empty it
-    XMLCh* tmpBuf = new XMLCh[XMLString::stringLen(basePath)
-                              + XMLString::stringLen(relativePath)
-                              + 2];
-    *tmpBuf = 0;
+    char  dirBuf[PATH_MAX + 1];
+    char  *curDir = getcwd(&dirBuf[0], PATH_MAX + 1);
 
-    //
-    //  If we have no base path, then just take the relative path as
-    //  is.
-    //
-    if (!basePath)
+    if (!curDir)
     {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetBasePathName);
     }
 
-    if (!*basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    const XMLCh* basePtr = basePath + (XMLString::stringLen(basePath) - 1);
-    if ((*basePtr != chForwardSlash)
-    &&  (*basePtr != chBackSlash))
-    {
-        while ((basePtr >= basePath)
-        &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-        {
-            basePtr--;
-        }
-    }
-
-    // There is no relevant base path, so just take the relative part
-    if (basePtr < basePath)
-    {
-        XMLString::copyString(tmpBuf, relativePath);
-        return tmpBuf;
-    }
-
-    // After this, make sure the buffer gets handled if we exit early
-    ArrayJanitor<XMLCh> janBuf(tmpBuf);
-
-    //
-    //  We have some path part, so we need to check to see if we ahve to
-    //  weave any of the parts together.
-    //
-    const XMLCh* pathPtr = relativePath;
-    while (true)
-    {
-        // If it does not start with some period, then we are done
-        if (*pathPtr != chPeriod)
-            break;
-
-        unsigned int periodCount = 1;
-        pathPtr++;
-        if (*pathPtr == chPeriod)
-        {
-            pathPtr++;
-            periodCount++;
-        }
-
-        // Has to be followed by a \ or / or the null to mean anything
-        if ((*pathPtr != chForwardSlash) && (*pathPtr != chBackSlash)
-        &&  *pathPtr)
-        {
-            break;
-        }
-        if (*pathPtr)
-            pathPtr++;
-
-        // If its one period, just eat it, else move backwards in the base
-        if (periodCount == 2)
-        {
-            basePtr--;
-            while ((basePtr >= basePath)
-            &&     ((*basePtr != chForwardSlash) && (*basePtr != chBackSlash)))
-            {
-                basePtr--;
-            }
-
-            // The base cannot provide enough levels, so its in error/
-            if (basePtr < basePath)
-                ThrowXML(XMLPlatformUtilsException,
-                         XMLExcepts::File_BasePathUnderflow);
-        }
-    }
-
-    // Copy the base part up to the base pointer
-    XMLCh* bufPtr = tmpBuf;
-    const XMLCh* tmpPtr = basePath;
-    while (tmpPtr <= basePtr)
-        *bufPtr++ = *tmpPtr++;
-
-    // And then copy on the rest of our path
-    XMLString::copyString(bufPtr, pathPtr);
-
-    // Orphan the buffer and return it
-    janBuf.orphan();
-    return tmpBuf;
+    return XMLString::transcode(curDir, manager);
 }
 
-
-
+inline bool XMLPlatformUtils::isAnySlash(XMLCh c) 
+{
+    return ( chBackSlash == c || chForwardSlash == c);
+}
 
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
@@ -542,13 +440,13 @@ void XMLPlatformUtils::platformInit()
     if (pthread_mutex_init(gAtomicOpMutex, pthread_mutexattr_default)) {
 	delete gAtomicOpMutex;
 	gAtomicOpMutex = 0;
-        panic( XMLPlatformUtils::Panic_SystemInit );
+        panic( PanicHandler::Panic_SystemInit );
     }
 #else // XML_USE_DCE
     if (pthread_mutex_init(gAtomicOpMutex, NULL)) {
 	delete gAtomicOpMutex;
 	gAtomicOpMutex = 0;
-        panic( XMLPlatformUtils::Panic_SystemInit );
+        panic( PanicHandler::Panic_SystemInit );
     }
 #endif // XML_USE_DCE
 }
@@ -679,14 +577,14 @@ void* XMLPlatformUtils::compareAndSwap ( void**      toFill ,
     // Currently its supported only in the kernel mode
 
     if (pthread_mutex_lock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     void *retVal = *toFill;
     if (*toFill == toCompare)
               *toFill = (void *)newValue;
 
     if (pthread_mutex_unlock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     return retVal;
 }
@@ -696,12 +594,12 @@ int XMLPlatformUtils::atomicIncrement(int &location)
     //return (int)atomic_add_32_nv( (uint32_t*)&location, 1);
 
     if (pthread_mutex_lock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     int tmp = ++location;
 
     if (pthread_mutex_unlock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     return tmp;
 }
@@ -710,12 +608,12 @@ int XMLPlatformUtils::atomicDecrement(int &location)
     //return (int)atomic_add_32_nv( (uint32_t*)&location, -1);
 
     if (pthread_mutex_lock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     int tmp = --location;
 
     if (pthread_mutex_unlock( gAtomicOpMutex))
-        panic(XMLPlatformUtils::Panic_SynchronizationErr);
+        panic(PanicHandler::Panic_SynchronizationErr);
 
     return tmp;
 }
@@ -769,7 +667,7 @@ int XMLPlatformUtils::atomicDecrement(int &location)
 
 FileHandle XMLPlatformUtils::openStdInHandle()
 {
-        return (FileHandle)fdopen(dup(0), "rb");
+        return (FileHandle)dup(0);
 }
 
 void XMLPlatformUtils::platformTerm()
@@ -781,4 +679,6 @@ void XMLPlatformUtils::platformTerm()
 #endif
 }
 
+#include <xercesc/util/LogicalPath.c>
+
 XERCES_CPP_NAMESPACE_END
