@@ -56,6 +56,21 @@
 
 /**
  * $Log: RefHashTableOf.c,v $
+ * Revision 1.11  2003/08/20 11:51:39  gareth
+ * Reorderd initializer list to prevent compiler warning.
+ *
+ * Revision 1.10  2003/05/18 14:02:05  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.9  2003/05/16 21:36:59  knoaman
+ * Memory manager implementation: Modify constructors to pass in the memory manager.
+ *
+ * Revision 1.8  2003/05/15 19:04:35  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.7  2003/05/15 10:37:08  gareth
+ * Optimization. We now resize the hash when appropriate. Patch by Nathan Codding.
+ *
  * Revision 1.6  2002/11/04 15:22:04  tng
  * C++ Namespace Support.
  *
@@ -129,30 +144,61 @@ XERCES_CPP_NAMESPACE_BEGIN
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Constructors and Destructor
 // ---------------------------------------------------------------------------
-template <class TVal> RefHashTableOf<TVal>::RefHashTableOf(const unsigned int modulus, const bool adoptElems)
-	: fAdoptedElems(adoptElems), fBucketList(0), fHashModulus(modulus)
+template <class TVal>
+RefHashTableOf<TVal>::RefHashTableOf( const unsigned int modulus
+                                    , const bool adoptElems
+                                    , MemoryManager* const manager)
+
+    : fMemoryManager(manager)
+    , fAdoptedElems(adoptElems)
+    , fBucketList(0)
+    , fHashModulus(modulus)
+    , fInitialModulus(modulus)
+    , fCount(0)
+    , fHash(0)
+
 {
     initialize(modulus);
 	
 	// create default hasher
-	fHash = new HashXMLCh();
+	fHash = new (fMemoryManager) HashXMLCh();
 }
 
-template <class TVal> RefHashTableOf<TVal>::RefHashTableOf(const unsigned int modulus, const bool adoptElems, HashBase* hashBase)
-	: fAdoptedElems(adoptElems), fBucketList(0), fHashModulus(modulus)
+template <class TVal>
+RefHashTableOf<TVal>::RefHashTableOf( const unsigned int modulus
+                                    , const bool adoptElems
+                                    , HashBase* hashBase
+                                    , MemoryManager* const manager)
+
+    : fMemoryManager(manager)
+    , fAdoptedElems(adoptElems)
+    , fBucketList(0)
+    , fHashModulus(modulus)
+    , fInitialModulus(modulus)
+    , fCount(0)
+    , fHash(0)
 {
     initialize(modulus);
     // set hasher
     fHash = hashBase;
 }
 
-template <class TVal> RefHashTableOf<TVal>::RefHashTableOf(const unsigned int modulus)
-	: fAdoptedElems(true), fBucketList(0), fHashModulus(modulus)
+template <class TVal>
+RefHashTableOf<TVal>::RefHashTableOf(const unsigned int modulus
+                                     , MemoryManager* const manager)
+
+    : fMemoryManager(manager)
+    , fAdoptedElems(true)
+    , fBucketList(0)
+    , fHashModulus(modulus)
+    , fInitialModulus(modulus)
+    , fCount(0)
+    , fHash(0)
 {
     initialize(modulus);
 
     // create default hasher
-    fHash = new HashXMLCh();
+    fHash = new (fMemoryManager) HashXMLCh();
 }
 
 template <class TVal> void RefHashTableOf<TVal>::initialize(const unsigned int modulus)
@@ -161,7 +207,10 @@ template <class TVal> void RefHashTableOf<TVal>::initialize(const unsigned int m
         ThrowXML(IllegalArgumentException, XMLExcepts::HshTbl_ZeroModulus);
 
     // Allocate the bucket list and zero them
-    fBucketList = new RefHashTableBucketElem<TVal>*[fHashModulus];
+    fBucketList = (RefHashTableBucketElem<TVal>**) fMemoryManager->allocate
+    (
+        fHashModulus * sizeof(RefHashTableBucketElem<TVal>*)
+    ); //new RefHashTableBucketElem<TVal>*[fHashModulus];
     for (unsigned int index = 0; index < fHashModulus; index++)
         fBucketList[index] = 0;
 }
@@ -171,7 +220,7 @@ template <class TVal> RefHashTableOf<TVal>::~RefHashTableOf()
     removeAll();
 
     // Then delete the bucket list & hasher
-    delete [] fBucketList;
+    fMemoryManager->deallocate(fBucketList); //delete [] fBucketList;
     delete fHash;
 }
 
@@ -233,6 +282,8 @@ template <class TVal> void RefHashTableOf<TVal>::removeAll()
         // Clean out this entry
         fBucketList[buckInd] = 0;
     }
+
+    fCount = 0;
 }
 
 // This method returns the data associated with a key. The key entry is deleted. The caller
@@ -299,7 +350,8 @@ template <class TElem> void RefHashTableOf<TElem>::cleanup()
     removeAll();
 
     // Then delete the bucket list & hasher
-    delete [] fBucketList;
+    fMemoryManager->deallocate(fBucketList); //delete [] fBucketList;
+    fBucketList = 0;
     delete fHash;
 }
 
@@ -313,12 +365,13 @@ template <class TElem> void RefHashTableOf<TElem>::reinitialize(HashBase* hashBa
     if (fBucketList || fHash)
         cleanup();
 
+    fHashModulus = fInitialModulus;
     initialize(fHashModulus);
 
     if (hashBase)
         fHash = hashBase;
     else
-        fHash = new HashXMLCh();   // create default hasher
+        fHash = new (fMemoryManager) HashXMLCh();   // create default hasher
 }
 
 
@@ -359,6 +412,12 @@ get(const void* const key) const
     return findIt->fData;
 }
 
+template <class TVal>
+MemoryManager* RefHashTableOf<TVal>::getMemoryManager() const
+{
+    return fMemoryManager;
+}
+
 
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Getters
@@ -374,6 +433,13 @@ void RefHashTableOf<TVal>::setAdoptElements(const bool aValue)
 // ---------------------------------------------------------------------------
 template <class TVal> void RefHashTableOf<TVal>::put(void* key, TVal* const valueToAdopt)
 {
+    // Apply 0.75 load factor to find threshold.
+    unsigned int threshold = fHashModulus * 3 / 4;
+    
+    // If we've grown too big, expand the table and rehash.
+    if (fCount >= threshold)
+        rehash();
+
     // First see if the key exists already
     unsigned int hashVal;
     RefHashTableBucketElem<TVal>* newBucket = findBucketElem(key, hashVal);
@@ -391,9 +457,11 @@ template <class TVal> void RefHashTableOf<TVal>::put(void* key, TVal* const valu
     }
      else
     {
-        newBucket = new RefHashTableBucketElem<TVal>(key, valueToAdopt, fBucketList[hashVal]);
+        newBucket = new (fMemoryManager) RefHashTableBucketElem<TVal>(key, valueToAdopt, fBucketList[hashVal]);
         fBucketList[hashVal] = newBucket;
     }
+
+    fCount++;
 }
 
 
@@ -401,6 +469,51 @@ template <class TVal> void RefHashTableOf<TVal>::put(void* key, TVal* const valu
 // ---------------------------------------------------------------------------
 //  RefHashTableOf: Private methods
 // ---------------------------------------------------------------------------
+template <class TVal> void RefHashTableOf<TVal>::rehash()
+{
+    unsigned int index;
+    unsigned int oldMod = fHashModulus;
+    fHashModulus *= 2;
+    
+    RefHashTableBucketElem<TVal>** oldBucketList = fBucketList;
+    
+    fBucketList = (RefHashTableBucketElem<TVal>**) fMemoryManager->allocate
+    (
+        fHashModulus * sizeof(RefHashTableBucketElem<TVal>*)
+    );//new RefHashTableBucketElem<TVal>*[fHashModulus];
+    for (index = 0; index < fHashModulus; index++)
+        fBucketList[index] = 0;
+    
+    
+    // Rehash all existing entries.
+    for (index = 0; index < oldMod; index++)
+    {
+        // Get the bucket list head for this entry
+        RefHashTableBucketElem<TVal>* curElem = oldBucketList[index];
+        RefHashTableBucketElem<TVal>* nextElem;
+        while (curElem)
+        {
+            // Save the next element before we detach this one
+            nextElem = curElem->fNext;
+
+            unsigned int hashVal = fHash->getHashVal(curElem->fKey, fHashModulus);
+            if (hashVal > fHashModulus)
+                ThrowXML(RuntimeException, XMLExcepts::HshTbl_BadHashFromKey);
+            
+            RefHashTableBucketElem<TVal>* newHeadElem = fBucketList[hashVal];
+            
+            // Insert at the start of this bucket's list.
+            curElem->fNext = newHeadElem;
+            fBucketList[hashVal] = curElem;
+            
+            curElem = nextElem;
+        }
+    }
+            
+    fMemoryManager->deallocate(oldBucketList);//delete[] oldBucketList;
+    
+}
+
 template <class TVal> RefHashTableBucketElem<TVal>* RefHashTableOf<TVal>::
 findBucketElem(const void* const key, unsigned int& hashVal)
 {
@@ -479,6 +592,8 @@ removeBucketElem(const void* const key, unsigned int& hashVal)
             // Delete the current element
             delete curElem;
 
+            fCount--;
+
             return;
         }
 
