@@ -1,7 +1,7 @@
 /*
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,18 +55,17 @@
  */
 
 /*
- * $Id: TransService.cpp,v 1.11 2003/01/24 22:05:35 tng Exp $
+ * $Id: TransService.cpp,v 1.14 2003/11/25 03:46:26 neilg Exp $
  */
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/util/Janitor.hpp>
-#include <xercesc/util/RefHashTableOf.hpp>
-#include <xercesc/util/RefVectorOf.hpp>
 #include <xercesc/util/XML88591Transcoder.hpp>
 #include <xercesc/util/XMLASCIITranscoder.hpp>
 #include <xercesc/util/XMLChTranscoder.hpp>
 #include <xercesc/util/XMLEBCDICTranscoder.hpp>
+#include <xercesc/util/XMLIBM1047Transcoder.hpp>
 #include <xercesc/util/XMLIBM1140Transcoder.hpp>
 #include <xercesc/util/XMLUCS4Transcoder.hpp>
 #include <xercesc/util/XMLUTF8Transcoder.hpp>
@@ -74,43 +73,35 @@
 #include <xercesc/util/XMLWin1252Transcoder.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
-#include <xercesc/util/TransENameMap.hpp>
 #include <xercesc/util/EncodingValidator.hpp>
 #include <xercesc/util/XMLRegisterCleanup.hpp>
 #include <xercesc/util/PlatformUtils.hpp>
+#include <xercesc/util/TransENameMap.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Local, static data
 //
-//  gMappings
-//      This is a hash table of ENameMap objects. It is created and filled
-//      in when the platform init calls our initTransService() method.
-//
-//  gMappingsRecognizer
-//      This is an array of ENameMap objects, predefined for those
-//      already recognized by XMLRecognizer::Encodings.
-//
 //  gStrictIANAEncoding
 //      A flag to control whether strict IANA encoding names checking should
 //      be done
 //
 // ---------------------------------------------------------------------------
-static RefHashTableOf<ENameMap>*    gMappings = 0;
-static RefVectorOf<ENameMap>*       gMappingsRecognizer = 0;
 static bool gStrictIANAEncoding = false;
+RefHashTableOf<ENameMap>* XMLTransService::gMappings = 0;
+RefVectorOf<ENameMap> * XMLTransService::gMappingsRecognizer = 0;
 
 // -----------------------------------------------------------------------
 //  Notification that lazy data has been deleted
 // -----------------------------------------------------------------------
-static void reinitMappings() {
+void XMLTransService::reinitMappings() {
     delete gMappings;    // The contents of the gMappings hash table are owned by
     gMappings = 0;       //   the it, and so will be deleted by gMapping's destructor.
 }
 
-static void reinitMappingsRecognizer() {
-    delete gMappingsRecognizer;
+void XMLTransService::reinitMappingsRecognizer() {
+    delete XMLTransService::gMappingsRecognizer;
     gMappingsRecognizer = 0;
 }
 
@@ -169,18 +160,20 @@ void XMLTransService::addEncoding(const XMLCh* const encoding,
 XMLTranscoder*
 XMLTransService::makeNewTranscoderFor(  const   char* const             encodingName
                                         ,       XMLTransService::Codes& resValue
-                                        , const unsigned int            blockSize)
+                                        , const unsigned int            blockSize
+                                        ,       MemoryManager* const    manager)
 {
-    XMLCh* tmpName = XMLString::transcode(encodingName);
-    ArrayJanitor<XMLCh> janName(tmpName);
+    XMLCh* tmpName = XMLString::transcode(encodingName, manager);
+    ArrayJanitor<XMLCh> janName(tmpName, manager);
 
-    return makeNewTranscoderFor(tmpName, resValue, blockSize);
+    return makeNewTranscoderFor(tmpName, resValue, blockSize, manager);
 }
 
 XMLTranscoder*
 XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encodingName
                                         ,       XMLTransService::Codes& resValue
-                                        , const unsigned int            blockSize)
+                                        , const unsigned int            blockSize
+                                        ,       MemoryManager* const    manager)
 {
     //
     // If strict IANA encoding flag is set, validate encoding name
@@ -213,7 +206,7 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
     // If we found it, then call the factory method for it
     if (ourMapping)
 	{		
-       XMLTranscoder* temp = ourMapping->makeNew(blockSize);
+       XMLTranscoder* temp = ourMapping->makeNew(blockSize, manager);
        resValue = temp ? XMLTransService::Ok : XMLTransService::InternalFailure;
        return temp;
     }
@@ -223,7 +216,7 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
     //  to the trans service to see if he can make anything of it.
     //
 
-    XMLTranscoder* temp =  makeNewXMLTranscoder(encodingName, resValue, blockSize);
+    XMLTranscoder* temp =  makeNewXMLTranscoder(encodingName, resValue, blockSize, manager);
 
     // if successful, set resValue to OK
     // if failed, the makeNewXMLTranscoder has already set the proper failing resValue
@@ -237,7 +230,8 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
 XMLTranscoder*
 XMLTransService::makeNewTranscoderFor(  XMLRecognizer::Encodings        encodingEnum
                                         ,       XMLTransService::Codes& resValue
-                                        , const unsigned int            blockSize)
+                                        , const unsigned int            blockSize
+                                        ,       MemoryManager* const    manager)
 {
     //
     // We can only make transcoder if the passed encodingEnum is under this range
@@ -251,12 +245,12 @@ XMLTransService::makeNewTranscoderFor(  XMLRecognizer::Encodings        encoding
 
     // If we found it, then call the factory method for it
     if (ourMapping)	{		
-       XMLTranscoder* temp = ourMapping->makeNew(blockSize);
+       XMLTranscoder* temp = ourMapping->makeNew(blockSize, manager);
        resValue = temp ? XMLTransService::Ok : XMLTransService::InternalFailure;
        return temp;
     }
     else {
-        XMLTranscoder* temp =  makeNewXMLTranscoder(XMLRecognizer::nameForEncoding(encodingEnum), resValue, blockSize);
+        XMLTranscoder* temp =  makeNewXMLTranscoder(XMLRecognizer::nameForEncoding(encodingEnum), resValue, blockSize, manager);
 
         // if successful, set resValue to OK
         // if failed, the makeNewXMLTranscoder has already set the proper failing resValue
@@ -497,6 +491,11 @@ void XMLTransService::initTransService()
     gMappings->put((void*)XMLUni::fgIBM037EncodingString, new ENameMapFor<XMLEBCDICTranscoder>(XMLUni::fgIBM037EncodingString));
     gMappings->put((void*)XMLUni::fgIBM037EncodingString2, new ENameMapFor<XMLEBCDICTranscoder>(XMLUni::fgIBM037EncodingString2));
 
+
+    //hhe
+    gMappings->put((void*)XMLUni::fgIBM1047EncodingString, new ENameMapFor<XMLIBM1047Transcoder>(XMLUni::fgIBM1047EncodingString));
+    gMappings->put((void*)XMLUni::fgIBM1047EncodingString2, new ENameMapFor<XMLIBM1047Transcoder>(XMLUni::fgIBM1047EncodingString2));
+
     //
     //  Add in our mappings for IBM037 with Euro update, i.e. IBM1140. It
     //  has alias IBM01140, the one suggested by IANA
@@ -532,7 +531,7 @@ bool XMLTransService::isStrictIANAEncoding()
 // ---------------------------------------------------------------------------
 XMLTranscoder::~XMLTranscoder()
 {
-    delete [] fEncodingName;
+    fMemoryManager->deallocate(fEncodingName);//delete [] fEncodingName;
 }
 
 
@@ -540,11 +539,13 @@ XMLTranscoder::~XMLTranscoder()
 //  XMLTranscoder: Hidden Constructors
 // ---------------------------------------------------------------------------
 XMLTranscoder::XMLTranscoder(const  XMLCh* const    encodingName
-                            , const unsigned int    blockSize) :
+                            , const unsigned int    blockSize
+                            , MemoryManager* const  manager) :
     fEncodingName(0)
     , fBlockSize(blockSize)
+    , fMemoryManager(manager)
 {
-    fEncodingName = XMLString::replicate(encodingName);
+    fEncodingName = XMLString::replicate(encodingName, fMemoryManager);
 }
 
 
