@@ -57,7 +57,7 @@
 /**
  * @01A D998714.1 V5R2M0    100301   Swan    :Fix error return flags
  * @02A           V5R2M0    200419   jrhansen : support lowercase function
- * $Id: Iconv400TransService.cpp,v 1.7 2003/01/06 21:47:48 tng Exp $
+ * $Id: Iconv400TransService.cpp,v 1.9 2003/05/17 16:32:17 knoaman Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -84,13 +84,16 @@ XERCES_CPP_NAMESPACE_BEGIN
 //  When XMLCh and ICU's UChar are not the same size, we have to do a temp
 //  conversion of all strings. These local helper methods make that easier.
 //
-static UChar* convertToUChar(   const   XMLCh* const    toConvert
-                                , const unsigned int    srcLen = 0)
+static UChar* convertToUChar( const   XMLCh* const toConvert
+                            , const unsigned int   srcLen = 0)
 {
     const unsigned int actualLen = srcLen
                                    ? srcLen : XMLString::stringLen(toConvert);
 
-    UChar* tmpBuf = new UChar[srcLen + 1];
+    UChar* tmpBuf = (UChar*) XMLPlatformUtils::fgMemoryManager->allocate
+    (
+        (srcLen + 1) * sizeof(UChar)
+    );//new UChar[srcLen + 1];
     const XMLCh* srcPtr = toConvert;
     UChar* outPtr = tmpBuf;
     while (*srcPtr)
@@ -316,7 +319,8 @@ XMLCh Iconv400TransService::toUnicodeLower(XMLCh comp1) const
 XMLTranscoder*
 Iconv400TransService::makeNewXMLTranscoder(  const   XMLCh* const            encodingName
                                         ,       XMLTransService::Codes& resValue
-                                        , const unsigned int            blockSize)
+                                        , const unsigned int            blockSize
+                                        ,       MemoryManager* const    manager)
 {
     UErrorCode uerr = U_ZERO_ERROR;
     UConverter* converter = ucnv_openU(encodingName, &uerr);
@@ -325,7 +329,7 @@ Iconv400TransService::makeNewXMLTranscoder(  const   XMLCh* const            enc
         resValue = XMLTransService::UnsupportedEncoding;
         return 0;
     }
-    return new Iconv400Transcoder(encodingName, converter, blockSize);
+    return new (manager) Iconv400Transcoder(encodingName, converter, blockSize, manager);
 }
 
 
@@ -334,18 +338,19 @@ Iconv400TransService::makeNewXMLTranscoder(  const   XMLCh* const            enc
 // ---------------------------------------------------------------------------
 //  IconvTranscoder: Constructors and Destructor
 // ---------------------------------------------------------------------------
-Iconv400Transcoder::Iconv400Transcoder(const  XMLCh* const        encodingName
-                            ,       UConverter* const   toAdopt
-                            , const unsigned int        blockSize) :
+Iconv400Transcoder::Iconv400Transcoder( const XMLCh* const         encodingName
+                                      ,       UConverter* const    toAdopt
+                                      , const unsigned int         blockSize
+                                      ,       MemoryManager* const manager) :
 
-    XMLTranscoder(encodingName, blockSize)
+    XMLTranscoder(encodingName, blockSize, manager)
     , fConverter(toAdopt)
     , fFixed(false)
     , fSrcOffsets(0)
 {
     // If there is a block size, then allocate our source offset array
     if (blockSize)
-        fSrcOffsets = new long[blockSize];
+        fSrcOffsets = (long*) manager->allocate(blockSize * sizeof(long));//new long[blockSize];
 
     // Remember if its a fixed size encoding
     fFixed = (ucnv_getMaxCharSize(fConverter) == ucnv_getMinCharSize(fConverter));
@@ -353,7 +358,7 @@ Iconv400Transcoder::Iconv400Transcoder(const  XMLCh* const        encodingName
 
 Iconv400Transcoder::~Iconv400Transcoder()
 {
-    delete [] fSrcOffsets;
+    getMemoryManager()->deallocate(fSrcOffsets);//delete [] fSrcOffsets;
 
     // If there is a converter, ask Iconv400 to clean it up
     if (fConverter)
@@ -391,7 +396,10 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
     if (sizeof(XMLCh) == sizeof(UChar))
         startTarget = (UChar*)toFill;
      else
-        startTarget = new UChar[maxChars];
+        startTarget = (UChar*) getMemoryManager()->allocate
+        (
+            maxChars * sizeof(UChar)
+        );//new UChar[maxChars];
     UChar* orgTarget = startTarget;
 
     //
@@ -415,7 +423,7 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
     if ((err != U_ZERO_ERROR) && (err != U_INDEX_OUTOFBOUNDS_ERROR))
     {
         if (orgTarget != (UChar*)toFill)
-            delete [] orgTarget;
+            getMemoryManager()->deallocate(orgTarget);//delete [] orgTarget;
 
         if (fFixed)
         {
@@ -493,7 +501,7 @@ Iconv400Transcoder::transcodeFrom(const  XMLByte* const          srcData
             *outPtr++ = XMLCh(*startTarget++);
 
         // And delete the temp buffer
-        delete [] orgTarget;
+        getMemoryManager()->deallocate(orgTarget);//delete [] orgTarget;
     }
 
     // Return the chars we put into the target buffer
@@ -525,7 +533,7 @@ Iconv400Transcoder::transcodeTo( const   XMLCh* const    srcData
         tmpBufPtr = convertToUChar(srcData, srcCount);
         srcPtr = tmpBufPtr;
     }
-    ArrayJanitor<UChar> janTmpBuf(tmpBufPtr);
+    ArrayJanitor<UChar> janTmpBuf(tmpBufPtr, XMLPlatformUtils::fgMemoryManager);
 
     //
     //  Set the appropriate callback so that it will either fail or use
@@ -769,6 +777,76 @@ char* Iconv400LCPTranscoder::transcode(const XMLCh* const toTranscode)
     return retBuf;
 }
 
+char* Iconv400LCPTranscoder::transcode(const XMLCh* const toTranscode,
+                                       MemoryManager* const manager)
+{
+    char* retBuf = 0;
+
+    // Check for a couple of special cases
+    if (!toTranscode)
+        return 0;
+
+    if (!*toTranscode)
+    {
+        retBuf = (char*) manager->allocate(sizeof(char));//new char[1];
+        retBuf[0] = 0;
+        return retBuf;
+    }
+
+    // Caculate a return buffer size not too big, but less likely to overflow
+    int32_t targetLen = (int32_t)(u_strlen(toTranscode) * 1.25);
+
+    // Allocate the return buffer
+    retBuf = (char*) manager->allocate((targetLen + 1) * sizeof(char));//new char[targetLen + 1];
+
+    // Lock now while we call the converter.
+    UErrorCode err = U_ZERO_ERROR;
+    int32_t targetCap;
+    {
+        XMLMutexLock lockConverter(&fMutex);
+
+        //Convert the Unicode string to char*
+        targetCap = ucnv_fromUChars
+        (
+            fConverter
+            , retBuf
+            , targetLen + 1
+            , toTranscode
+            , &err
+        );
+    }
+
+    // If targetLen is not enough then buffer overflow might occur
+    if (err == U_BUFFER_OVERFLOW_ERROR)
+    {
+        // Reset the error, delete the old buffer, allocate a new one, and try again
+        err = U_ZERO_ERROR;
+        manager->deallocate(retBuf);//delete [] retBuf;
+        retBuf = (char*) manager->allocate(targetCap * sizeof(char));//new char[targetCap];
+
+        // Lock again before we retry
+        XMLMutexLock lockConverter(&fMutex);
+        targetCap = ucnv_fromUChars
+        (
+            fConverter
+            , retBuf
+            , targetCap
+            , toTranscode
+            , &err
+        );
+    }
+
+    if (U_FAILURE(err))
+    {
+        manager->deallocate(retBuf);//delete [] retBuf;
+        return 0;
+    }
+
+    // Cap it off and return
+    retBuf[targetCap] = 0;
+    return retBuf;
+}
+
 XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode)
 {
     // Watch for a few pyscho corner cases
@@ -840,6 +918,78 @@ XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode)
     return retVal;
 }
 
+XMLCh* Iconv400LCPTranscoder::transcode(const char* const toTranscode,
+                                        MemoryManager* const manager)
+{
+    // Watch for a few pyscho corner cases
+    if (!toTranscode)
+        return 0;
+
+    XMLCh* retVal = 0;
+    if (!*toTranscode)
+    {
+        retVal = (XMLCh*) manager->allocate(sizeof(XMLCh));//new XMLCh[1];
+        retVal[0] = 0;
+        return retVal;
+    }
+
+    //
+    //  Get the length of the string to transcode. The Unicode string will
+    //  almost always be no more chars than were in the source, so this is
+    //  the best guess as to the storage needed.
+    //
+    const int32_t srcLen = (int32_t)strlen(toTranscode);
+    // Allocate unicode string of equivalent length in unicode bytes
+    retVal = (XMLCh*) manager->allocate((srcLen + 1) * sizeof(XMLCh));//new XMLCh[srcLen+1];
+
+    // Now lock while we do these calculations
+    UErrorCode err = U_ZERO_ERROR;
+    {
+        XMLMutexLock lockConverter(&fMutex);
+
+        //
+        //  Here we don't know what the target length will be so use 0 and
+        //  expect an U_BUFFER_OVERFLOW_ERROR in which case it'd get resolved
+        //  by the correct capacity value.
+        //
+        int32_t targetCap;
+        targetCap = ucnv_toUChars
+        (
+            fConverter
+            , retVal
+            , srcLen+1
+            , toTranscode
+            , srcLen
+            , &err
+        );
+
+        if (err != U_BUFFER_OVERFLOW_ERROR)
+	{
+
+        err = U_ZERO_ERROR;
+        retVal = (XMLCh*) manager->allocate(targetCap * sizeof(XMLCh));//new XMLCh[targetCap];
+        ucnv_toUChars
+        (
+            fConverter
+            , retVal
+            , targetCap
+            , toTranscode
+            , srcLen
+            , &err
+           );
+	 }
+   }
+
+    if (U_FAILURE(err))
+    {
+        // Clean up if we got anything allocated
+        manager->deallocate(retVal);//delete [] retVal;
+        return 0;
+    }
+
+    return retVal;
+}
+
 
 bool Iconv400LCPTranscoder::transcode(const  char* const     toTranscode
                                 ,       XMLCh* const    toFill
