@@ -56,7 +56,7 @@
 
 /**
  * @02A           V5R2M0    200419   jrhansen : support lowercase function
- * $Id: Iconv400TransService.hpp,v 1.5 2003/01/06 21:47:48 tng Exp $
+ * $Id: Iconv400TransService.hpp,v 1.8 2003/05/17 16:32:17 knoaman Exp $
  */
 
 #ifndef ICONV400TRANSSERVICE_HPP
@@ -122,6 +122,7 @@ protected :
         const   XMLCh* const            encodingName
         ,       XMLTransService::Codes& resValue
         , const unsigned int            blockSize
+        ,       MemoryManager* const    manager
     );
 
 
@@ -130,7 +131,7 @@ private :
     //  Unimplemented constructors and operators
     // -----------------------------------------------------------------------
     Iconv400TransService(const Iconv400TransService&);
-    void operator=(const Iconv400TransService&);
+    Iconv400TransService& operator=(const Iconv400TransService&);
 
     XMLCh toUnicodeUpper(XMLCh toupper) const;
     XMLCh toUnicodeLower(XMLCh tolower) const;
@@ -150,9 +151,10 @@ public :
     // -----------------------------------------------------------------------
     Iconv400Transcoder
     (
-        const   XMLCh* const        encodingName
-        ,       UConverter* const   toAdopt
-        , const unsigned int        blockSize
+        const   XMLCh* const      encodingName
+        ,       UConverter* const toAdopt
+        , const unsigned int      blockSize
+        , MemoryManager* const    manager = XMLPlatformUtils::fgMemoryManager
     );
     ~Iconv400Transcoder();
 
@@ -192,7 +194,7 @@ private :
     // -----------------------------------------------------------------------
     Iconv400Transcoder();
     Iconv400Transcoder(const Iconv400Transcoder&);
-    void operator=(const Iconv400Transcoder&);
+    Iconv400Transcoder& operator=(const Iconv400Transcoder&);
 
 
     // -----------------------------------------------------------------------
@@ -237,8 +239,12 @@ public :
     virtual unsigned int calcRequiredSize(const XMLCh* const srcText);
 
     virtual char* transcode(const XMLCh* const toTranscode);
+    virtual char* transcode(const XMLCh* const toTranscode,
+                            MemoryManager* const manager);
 
     virtual XMLCh* transcode(const char* const toTranscode);
+    virtual XMLCh* transcode(const char* const toTranscode,
+                             MemoryManager* const manager);
 
     virtual bool transcode
     (
@@ -262,7 +268,7 @@ private :
     // -----------------------------------------------------------------------
     Iconv400LCPTranscoder();
     Iconv400LCPTranscoder(const Iconv400LCPTranscoder&);
-    void operator=(const Iconv400LCPTranscoder&);
+    Iconv400LCPTranscoder& operator=(const Iconv400LCPTranscoder&);
 
 
     // -----------------------------------------------------------------------
