@@ -55,8 +55,38 @@
  */
 
 /*
- * $Id: XMLDateTime.cpp,v 1.6 2003/02/02 23:54:43 peiyongz Exp $
+ * $Id: XMLDateTime.cpp,v 1.16 2003/09/25 22:24:28 peiyongz Exp $
  * $Log: XMLDateTime.cpp,v $
+ * Revision 1.16  2003/09/25 22:24:28  peiyongz
+ * Using writeString/readString
+ *
+ * Revision 1.15  2003/09/25 15:22:54  peiyongz
+ * Solve HP complier error
+ *
+ * Revision 1.14  2003/09/23 18:16:07  peiyongz
+ * Inplementation for Serialization/Deserialization
+ *
+ * Revision 1.13  2003/08/14 02:57:27  knoaman
+ * Code refactoring to improve performance of validation.
+ *
+ * Revision 1.12  2003/05/22 02:10:52  knoaman
+ * Default the memory manager.
+ *
+ * Revision 1.11  2003/05/18 14:02:05  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.10  2003/05/15 19:07:46  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
+ * Revision 1.9  2003/05/15 16:32:19  gareth
+ * We did not allow dateTimes with a timezone due to the last seconds fix.
+ *
+ * Revision 1.8  2003/03/23 22:54:49  peiyongz
+ * invalid second values
+ *
+ * Revision 1.7  2003/02/22 22:49:09  peiyongz
+ * Schema Errata E2-45 24:00:00 allowed
+ *
  * Revision 1.6  2003/02/02 23:54:43  peiyongz
  * getFormattedString() added to return original and converted value.
  *
@@ -244,8 +274,8 @@ int XMLDateTime::compare(const XMLDateTime* const pDate1
         return EQUAL;
 
     //long comparison algorithm is required
-    XMLDateTime tempA, *pTempA = &tempA;
-    XMLDateTime tempB, *pTempB = &tempB;
+    XMLDateTime tempA(XMLPlatformUtils::fgMemoryManager), *pTempA = &tempA;
+    XMLDateTime tempB(XMLPlatformUtils::fgMemoryManager), *pTempB = &tempB;
 
     addDuration(pTempA, pDate1, 0);
     addDuration(pTempB, pDate2, 0);
@@ -458,17 +488,26 @@ int XMLDateTime::compareOrder(const XMLDateTime* const lValue
 XMLDateTime::~XMLDateTime()
 {
     if (fBuffer)
-        delete[] fBuffer;
+        fMemoryManager->deallocate(fBuffer);//delete[] fBuffer;
 }
 
-XMLDateTime::XMLDateTime()
-:fBuffer(0)
+XMLDateTime::XMLDateTime(MemoryManager* const manager)
+: fStart(0)
+, fEnd(0)
+, fBufferMaxLen(0)
+, fBuffer(0)
+, fMemoryManager(manager)
 {
     reset();
 }
 
-XMLDateTime::XMLDateTime(const XMLCh* const aString)
-:fBuffer(0)
+XMLDateTime::XMLDateTime(const XMLCh* const aString,
+                         MemoryManager* const manager)
+: fStart(0)
+, fEnd(0)
+, fBufferMaxLen(0)
+, fBuffer(0)
+, fMemoryManager(manager)
 {
     setBuffer(aString);
 }
@@ -478,7 +517,9 @@ XMLDateTime::XMLDateTime(const XMLCh* const aString)
 // -----------------------------------------------------------------------
 
 XMLDateTime::XMLDateTime(const XMLDateTime &toCopy)
-:fBuffer(0)
+: fBufferMaxLen(0)
+, fBuffer(0)
+, fMemoryManager(toCopy.fMemoryManager)
 {
     copy(toCopy);
 }
@@ -506,6 +547,7 @@ XMLCh*  XMLDateTime::toString() const
 {
     assertBuffer();
 
+    // Return data using global operator new
     XMLCh* retBuf = XMLString::replicate(fBuffer);
     return retBuf;
 }
@@ -1041,6 +1083,13 @@ void XMLDateTime::getTime()
             fValue[MiliSecond] = parseInt(fStart, sign);  //get ms between UTC sign and fEnd
         }
 	}
+    else if(sign == 0 || sign != fStart)
+    {
+        // seconds has more than 2 digits
+        ThrowXML1(SchemaDateTimeException
+                , XMLExcepts::DateTime_min_invalid
+                , fBuffer);
+    }
 
     //parse UTC time zone (hh:mm)
     if ( sign > 0 ) {
@@ -1255,7 +1304,7 @@ void XMLDateTime::validateDateTime() const
 
     //validate hours
     if ((fValue[Hour] < 0)  ||
-        (fValue[Hour] > 23) ||
+        (fValue[Hour] > 24) ||
         ((fValue[Hour] == 24) && ((fValue[Minute] !=0) ||
                                   (fValue[Second] !=0) ||
                                   (fValue[MiliSecond] !=0))))
@@ -1344,15 +1393,16 @@ int XMLDateTime::findUTCSign (const int start)
 //
 int XMLDateTime::parseInt(const int start, const int end) const
 {
+    unsigned int retVal = 0;
+    for (int i=start; i < end; i++) {
 
-    XMLCh* strToScan = new XMLCh[end - start + 1];
-    ArrayJanitor<XMLCh>  jname(strToScan);
-    XMLString::subString(strToScan, fBuffer, start, end);
+        if (fBuffer[i] < chDigit_0 || fBuffer[i] > chDigit_9)
+            break;
 
-    unsigned int retVal;
-    XMLString::textToBin(strToScan, retVal);
+        retVal = (retVal * 10) + (unsigned int) (fBuffer[i] - chDigit_0);
+    }
 
-    return (int) retVal;
+    return (int) retVal;;
 }
 
 //
@@ -1390,4 +1440,56 @@ int XMLDateTime::parseIntYear(const int end) const
     return ( negative ? (-1) * yearVal : yearVal );
 }
 
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(XMLDateTime)
+
+void XMLDateTime::serialize(XSerializeEngine& serEng)
+{
+    //REVISIT: may not need to call base since it does nothing
+    XMLNumber::serialize(serEng);
+
+    int i = 0;
+
+    if (serEng.isStoring())
+    {
+        for (i = 0; i < TOTAL_SIZE; i++)
+        {
+            serEng<<fValue[i];
+        }
+
+        for (i = 0; i < TIMEZONE_ARRAYSIZE; i++)
+        {
+            serEng<<fTimeZone[i];
+        }
+
+        serEng<<fStart;
+        serEng<<fEnd;
+
+        serEng.writeString(fBuffer, fBufferMaxLen, XSerializeEngine::toWriteBufferLen);
+    }
+    else
+    {
+        for (i = 0; i < TOTAL_SIZE; i++)
+        {
+            serEng>>fValue[i];
+        }
+
+        for (i = 0; i < TIMEZONE_ARRAYSIZE; i++)
+        {
+            serEng>>fTimeZone[i];
+        }
+
+        serEng>>fStart;
+        serEng>>fEnd;
+
+        int dataLen = 0;
+        serEng.readString(fBuffer, fBufferMaxLen, dataLen ,XSerializeEngine::toReadBufferLen);
+
+    }
+
+}
+
 XERCES_CPP_NAMESPACE_END
