@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Match.hpp,v 1.3 2002/12/18 13:01:02 gareth Exp $
+ * $Id: Match.hpp,v 1.6 2003/05/22 02:10:52 knoaman Exp $
  */
 
 #if !defined(MATCH_HPP)
@@ -64,6 +64,7 @@
 // ---------------------------------------------------------------------------
 //  Includes
 // ---------------------------------------------------------------------------
+#include <xercesc/util/PlatformUtils.hpp>
 #include <xercesc/util/ArrayIndexOutOfBoundsException.hpp>
 #include <xercesc/util/RuntimeException.hpp>
 
@@ -72,13 +73,14 @@ XERCES_CPP_NAMESPACE_BEGIN
 /**
   * An instance of this class has ranges captured in matching
   */
-class XMLUTIL_EXPORT Match {
+  class XMLUTIL_EXPORT Match : public XMemory
+{
 public:
 
 	// -----------------------------------------------------------------------
   //  Public Constructors and Destructor
   // -----------------------------------------------------------------------
-	Match();
+	Match(MemoryManager* const manager = XMLPlatformUtils::fgMemoryManager);
 	
   /**
   * Copy constructor
@@ -130,6 +132,7 @@ private:
 	int fPositionsSize;
 	int* fStartPositions;
 	int* fEndPositions;
+    MemoryManager* fMemoryManager;
 };
 
 /**
