@@ -55,8 +55,26 @@
  */
 
 /*
- * $Id: DayDatatypeValidator.cpp,v 1.3 2002/12/18 14:17:55 gareth Exp $
+ * $Id: DayDatatypeValidator.cpp,v 1.9 2003/11/06 15:30:07 neilg Exp $
  * $Log: DayDatatypeValidator.cpp,v $
+ * Revision 1.9  2003/11/06 15:30:07  neilg
+ * first part of PSVI/schema component model implementation, thanks to David Cargill.  This covers setting the PSVIHandler on parser objects, as well as implementing XSNotation, XSSimpleTypeDefinition, XSIDCDefinition, and most of XSWildcard, XSComplexTypeDefinition, XSElementDeclaration, XSAttributeDeclaration and XSAttributeUse.
+ *
+ * Revision 1.8  2003/10/02 19:21:06  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.7  2003/10/01 16:32:41  neilg
+ * improve handling of out of memory conditions, bug #23415.  Thanks to David Cargill.
+ *
+ * Revision 1.6  2003/08/14 03:00:11  knoaman
+ * Code refactoring to improve performance of validation.
+ *
+ * Revision 1.5  2003/05/18 14:02:07  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
+ * Revision 1.4  2003/05/15 18:53:26  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.3  2002/12/18 14:17:55  gareth
  * Fix to bug #13438. When you eant a vector that calls delete[] on its members you should use RefArrayVectorOf.
  *
@@ -81,22 +99,26 @@
 //  Includes
 // ---------------------------------------------------------------------------
 #include <xercesc/validators/datatype/DayDatatypeValidator.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  Constructors and Destructor
 // ---------------------------------------------------------------------------
-DayDatatypeValidator::DayDatatypeValidator()
-:DateTimeValidator(0, 0, 0, DatatypeValidator::Day)
-{}
+DayDatatypeValidator::DayDatatypeValidator(MemoryManager* const manager)
+:DateTimeValidator(0, 0, 0, DatatypeValidator::Day, manager)
+{
+    setOrdered(XSSimpleTypeDefinition::ORDERED_PARTIAL);
+}
 
 DayDatatypeValidator::DayDatatypeValidator(
                           DatatypeValidator*            const baseValidator
                         , RefHashTableOf<KVStringPair>* const facets
-                        , RefArrayVectorOf<XMLCh>*           const enums
-                        , const int                           finalSet)
-:DateTimeValidator(baseValidator, facets, finalSet, DatatypeValidator::Day)
+                        , RefArrayVectorOf<XMLCh>*      const enums
+                        , const int                           finalSet
+                        , MemoryManager* const                manager)
+:DateTimeValidator(baseValidator, facets, finalSet, DatatypeValidator::Day, manager)
 {
     init(enums);
 }
@@ -104,12 +126,15 @@ DayDatatypeValidator::DayDatatypeValidator(
 DayDatatypeValidator::~DayDatatypeValidator()
 {}
 
-DatatypeValidator* DayDatatypeValidator::newInstance(
-                                      RefHashTableOf<KVStringPair>* const facets
-                                    , RefArrayVectorOf<XMLCh>*           const enums
-                                    , const int                           finalSet)
+DatatypeValidator* DayDatatypeValidator::newInstance
+(
+      RefHashTableOf<KVStringPair>* const facets
+    , RefArrayVectorOf<XMLCh>* const      enums
+    , const int                           finalSet
+    , MemoryManager* const                manager
+)
 {
-    return (DatatypeValidator*) new DayDatatypeValidator(this, facets, enums, finalSet);
+    return (DatatypeValidator*) new (manager) DayDatatypeValidator(this, facets, enums, finalSet, manager);
 }
 
 //
@@ -117,12 +142,16 @@ DatatypeValidator* DayDatatypeValidator::newInstance(
 //
 XMLDateTime* DayDatatypeValidator::parse(const XMLCh* const content)
 {
-    XMLDateTime *pRetDate = new XMLDateTime(content);
+    XMLDateTime *pRetDate = new (fMemoryManager) XMLDateTime(content, fMemoryManager);
 
     try
     {
         pRetDate->parseDay();
     }
+    catch(const OutOfMemoryException&)
+    {
+        throw;
+    }
     catch (...)
     {
         delete pRetDate;
@@ -132,6 +161,22 @@ XMLDateTime* DayDatatypeValidator::parse(const XMLCh* const content)
     return pRetDate;
 }
 
+void DayDatatypeValidator::parse(XMLDateTime* const pDate)
+{
+    pDate->parseDay();
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(DayDatatypeValidator)
+
+void DayDatatypeValidator::serialize(XSerializeEngine& serEng)
+{
+    DateTimeValidator::serialize(serEng);
+}
+
 XERCES_CPP_NAMESPACE_END
 
 /**
