@@ -56,6 +56,24 @@
 
 /*
  * $Log: SchemaAttDefList.cpp,v $
+ * Revision 1.8  2003/11/13 23:20:47  peiyongz
+ * initSize
+ *
+ * Revision 1.7  2003/11/10 21:54:51  neilg
+ * implementation for new stateless means of traversing attribute definition lists
+ *
+ * Revision 1.6  2003/10/20 11:46:28  gareth
+ * Pass in memory manager to constructors and use for creation of enumerators.
+ *
+ * Revision 1.5  2003/10/17 21:17:12  peiyongz
+ * using XTemplateSerializer
+ *
+ * Revision 1.4  2003/10/10 16:25:40  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.3  2003/05/18 14:02:07  knoaman
+ * Memory manager implementation: pass per instance manager.
+ *
  * Revision 1.2  2002/11/04 14:49:41  tng
  * C++ Namespace Support.
  *
@@ -76,22 +94,31 @@
 // ---------------------------------------------------------------------------
 #include <xercesc/validators/schema/SchemaAttDefList.hpp>
 
+#include <xercesc/internal/XTemplateSerializer.hpp>
+#include <xercesc/util/ArrayIndexOutOfBoundsException.hpp>
+
 XERCES_CPP_NAMESPACE_BEGIN
 
 // ---------------------------------------------------------------------------
 //  SchemaAttDefList: Constructors and Destructor
 // ---------------------------------------------------------------------------
-SchemaAttDefList::SchemaAttDefList(RefHash2KeysTableOf<SchemaAttDef>* const listToUse) :
-
-    fEnum(0)
-    , fList(listToUse)
+SchemaAttDefList::SchemaAttDefList(RefHash2KeysTableOf<SchemaAttDef>* const listToUse, MemoryManager* const manager)
+: XMLAttDefList(manager)
+,fEnum(0)
+,fList(listToUse)
+,fArray(0)
+,fCount(0)
+,fSize(0)
 {
-    fEnum = new RefHash2KeysTableOfEnumerator<SchemaAttDef>(listToUse);
+    fEnum = new (getMemoryManager()) RefHash2KeysTableOfEnumerator<SchemaAttDef>(listToUse);
+    fArray = (SchemaAttDef **)((getMemoryManager())->allocate( sizeof(SchemaAttDef*) << 1));
+    fSize = 2;
 }
 
 SchemaAttDefList::~SchemaAttDefList()
 {
     delete fEnum;
+    (getMemoryManager())->deallocate(fArray);
 }
 
 
@@ -113,7 +140,7 @@ bool SchemaAttDefList::isEmpty() const
 XMLAttDef* SchemaAttDefList::findAttDef(const  unsigned long   uriID
                                     , const XMLCh* const    attName)
 {
-   QName tempAtt(attName, uriID);
+   QName tempAtt(attName, uriID, XMLPlatformUtils::fgMemoryManager);
    return fList->get((void*)tempAtt.getLocalPart(), uriID);
 }
 
@@ -122,7 +149,7 @@ const XMLAttDef*
 SchemaAttDefList::findAttDef(  const   unsigned long   uriID
                             , const XMLCh* const    attName) const
 {
-   QName tempAtt(attName, uriID);
+   QName tempAtt(attName, uriID, XMLPlatformUtils::fgMemoryManager);
    return fList->get((void*)tempAtt.getLocalPart(), uriID);
 }
 
@@ -157,4 +184,94 @@ void SchemaAttDefList::Reset()
     fEnum->Reset();
 }
 
+/**
+ * return total number of attributes in this list
+ */
+unsigned int SchemaAttDefList::getAttDefCount() const
+{
+    return fCount;
+}
+
+/**
+ * return attribute at the index-th position in the list.
+ */
+XMLAttDef &SchemaAttDefList::getAttDef(unsigned int index) 
+{
+    if(index >= fCount)
+        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::AttrList_BadIndex);
+    return *(fArray[index]);
+}
+
+/**
+ * return attribute at the index-th position in the list.
+ */
+const XMLAttDef &SchemaAttDefList::getAttDef(unsigned int index) const 
+{
+    if(index >= fCount)
+        ThrowXML(ArrayIndexOutOfBoundsException, XMLExcepts::AttrList_BadIndex);
+    return *(fArray[index]);
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(SchemaAttDefList)
+
+void SchemaAttDefList::serialize(XSerializeEngine& serEng)
+{
+
+    XMLAttDefList::serialize(serEng);
+
+    if (serEng.isStoring())
+    {
+        /***
+         *
+         * Serialize RefHash2KeysTableOf<SchemaAttDef>
+         *
+         ***/
+        XTemplateSerializer::storeObject(fList, serEng);
+        serEng << fCount;
+
+        // do not serialize fEnum
+    }
+    else
+    {
+        /***
+         *
+         * Deserialize RefHash2KeysTableOf<SchemaAttDef>           
+         *
+         ***/
+        XTemplateSerializer::loadObject(&fList, 29, true, serEng);
+
+        // assume empty so we can size fArray just right
+        serEng >> fSize;
+        if (!fEnum && fList)
+        {
+            fEnum = new (getMemoryManager()) RefHash2KeysTableOfEnumerator<SchemaAttDef>(fList);
+        }
+        if(fSize) 
+        {
+            (getMemoryManager())->deallocate(fArray);
+            fArray = (SchemaAttDef **)((getMemoryManager())->allocate( sizeof(SchemaAttDef*) * fSize));
+            fCount = 0;
+            while(fEnum->hasMoreElements())
+            {
+                fArray[fCount++] = &fEnum->nextElement();
+            }
+        }
+    }
+
+}
+
+SchemaAttDefList::SchemaAttDefList(MemoryManager* const manager)
+:fEnum(0)
+,fList(0)
+,fArray(0)
+,fCount(0)
+,fSize(0)
+{
+}
+
 XERCES_CPP_NAMESPACE_END
+
