@@ -56,6 +56,12 @@
 
 /*
  * $Log: IC_Selector.cpp,v $
+ * Revision 1.6  2003/10/14 15:24:23  peiyongz
+ * Implementation of Serialization/Deserialization
+ *
+ * Revision 1.5  2003/05/15 18:59:34  knoaman
+ * Partial implementation of the configurable memory manager.
+ *
  * Revision 1.4  2003/01/13 16:30:19  knoaman
  * [Bug 14469] Validator doesn't enforce xsd:key.
  *
@@ -96,8 +102,9 @@ XERCES_CPP_NAMESPACE_BEGIN
 SelectorMatcher::SelectorMatcher(XercesXPath* const xpath,
                                  IC_Selector* const selector,
                                  FieldActivator* const fieldActivator,
-                                 const int initialDepth)
-    : XPathMatcher(xpath, selector->getIdentityConstraint())
+                                 const int initialDepth,
+                                 MemoryManager* const manager)
+    : XPathMatcher(xpath, selector->getIdentityConstraint(), manager)
     , fInitialDepth(initialDepth)
     , fElementDepth(0)
     , fMatchedDepth(-1)
@@ -188,9 +195,40 @@ bool IC_Selector::operator !=(const IC_Selector& other) const {
 // ---------------------------------------------------------------------------
 //  IC_Selector: Factory methods
 // ---------------------------------------------------------------------------
-XPathMatcher* IC_Selector::createMatcher(FieldActivator* const fieldActivator, const int initialDepth) {
+XPathMatcher* IC_Selector::createMatcher(FieldActivator* const fieldActivator,
+                                         const int initialDepth,
+                                         MemoryManager* const manager) {
 
-    return new SelectorMatcher(fXPath, this, fieldActivator, initialDepth);
+    return new (manager) SelectorMatcher(fXPath, this, fieldActivator, initialDepth, manager);
+}
+
+/***
+ * Support for Serialization/De-serialization
+ ***/
+
+IMPL_XSERIALIZABLE_TOCREATE(IC_Selector)
+
+void IC_Selector::serialize(XSerializeEngine& serEng)
+{
+    if (serEng.isStoring())
+    {
+        serEng<<fXPath;
+        
+        IdentityConstraint::storeIC(serEng, fIdentityConstraint);
+    }
+    else
+    {
+        serEng>>fXPath;
+
+        fIdentityConstraint = IdentityConstraint::loadIC(serEng);
+    }
+
+}
+
+IC_Selector::IC_Selector(MemoryManager* const )
+:fXPath(0)
+,fIdentityConstraint(0)
+{
 }
 
 XERCES_CPP_NAMESPACE_END
